@@ -21,9 +21,7 @@ static guint update_id;
 #define UPDATE_INTERVAL 500
 
 #define THREAD
-
-// now deprecated: 
-//#define PAD_SEEK
+#define PAD_SEEK
 
 typedef struct
 {
@@ -61,7 +59,6 @@ setup_dynamic_link (GstElement *element, const gchar *padname, GstPad *target, G
   g_signal_connect (G_OBJECT (element), "new_pad", G_CALLBACK (dynamic_link), connect);
 }
 
-/* A MOD Audio decoder pipe. */
 static GstElement*
 make_mod_pipeline (const gchar *location) 
 {
@@ -90,12 +87,9 @@ make_mod_pipeline (const gchar *location)
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (decoder, "sink"));
 
-  seekable_elements = g_list_prepend (seekable_elements, audiosink);
-
   return pipeline;
 }
 
-/* A DV A/V decoder pipe. */
 static GstElement*
 make_dv_pipeline (const gchar *location) 
 {
@@ -130,13 +124,9 @@ make_dv_pipeline (const gchar *location)
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (decoder, "sink"));
 
-  seekable_elements = g_list_prepend (seekable_elements, audiosink);
-  seekable_elements = g_list_prepend (seekable_elements, videosink);
-
   return pipeline;
 }
 
-/* A WAV audio decoder pipe. */
 static GstElement*
 make_wav_pipeline (const gchar *location) 
 {
@@ -165,12 +155,9 @@ make_wav_pipeline (const gchar *location)
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (decoder, "sink"));
 
-  seekable_elements = g_list_prepend (seekable_elements, audiosink);
-
   return pipeline;
 }
 
-/* A FLAC decoder pipe. */
 static GstElement*
 make_flac_pipeline (const gchar *location) 
 {
@@ -183,7 +170,7 @@ make_flac_pipeline (const gchar *location)
   src = gst_element_factory_make_or_warn (SOURCE, "src");
   decoder = gst_element_factory_make_or_warn ("flacdec", "decoder");
   audiosink = gst_element_factory_make_or_warn ("osssink", "sink");
-  //g_object_set (G_OBJECT (audiosink), "sync", FALSE, NULL);
+  g_object_set (G_OBJECT (audiosink), "sync", FALSE, NULL);
 
   g_object_set (G_OBJECT (src), "location", location, NULL);
 
@@ -199,12 +186,9 @@ make_flac_pipeline (const gchar *location)
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (decoder, "sink"));
 
-  seekable_elements = g_list_prepend (seekable_elements, audiosink);
-
   return pipeline;
 }
 
-/* A SID audio decoder pipe. */
 static GstElement*
 make_sid_pipeline (const gchar *location) 
 {
@@ -233,12 +217,9 @@ make_sid_pipeline (const gchar *location)
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (decoder, "sink"));
 
-  seekable_elements = g_list_prepend (seekable_elements, audiosink);
-
   return pipeline;
 }
 
-/* An MPEG parser pipeline, to get info on an MPEG file. */
 static GstElement*
 make_parse_pipeline (const gchar *location) 
 {
@@ -268,12 +249,9 @@ make_parse_pipeline (const gchar *location)
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (parser, "sink"));
 
-  seekable_elements = g_list_prepend (seekable_elements, fakesink);
-
   return pipeline;
 }
 
-/* A Vorbis Audio decoder pipe. */
 static GstElement*
 make_vorbis_pipeline (const gchar *location) 
 {
@@ -286,7 +264,7 @@ make_vorbis_pipeline (const gchar *location)
   src = gst_element_factory_make_or_warn (SOURCE, "src");
   decoder = gst_element_factory_make_or_warn ("vorbisfile", "decoder");
   audiosink = gst_element_factory_make_or_warn ("osssink", "sink");
-  g_object_set (G_OBJECT (audiosink), "sync", FALSE, NULL);
+  g_object_set (G_OBJECT (audiosink), "sync", TRUE, NULL);
 
   g_object_set (G_OBJECT (src), "location", location, NULL);
 
@@ -302,12 +280,9 @@ make_vorbis_pipeline (const gchar *location)
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (decoder, "sink"));
 
-  seekable_elements = g_list_prepend (seekable_elements, audiosink);
-
   return pipeline;
 }
 
-/* An MP3 Audio decoder pipe. */
 static GstElement*
 make_mp3_pipeline (const gchar *location) 
 {
@@ -347,7 +322,6 @@ make_mp3_pipeline (const gchar *location)
   return pipeline;
 }
 
-/* An AVI A/V decoder pipe. */
 static GstElement*
 make_avi_pipeline (const gchar *location) 
 {
@@ -392,7 +366,7 @@ make_avi_pipeline (const gchar *location)
   video_bin = gst_bin_new ("v_decoder_bin");
   //v_decoder = gst_element_factory_make_or_warn ("identity", "v_dec");
   //v_decoder = gst_element_factory_make_or_warn ("windec", "v_dec");
-  v_decoder = gst_element_factory_make_or_warn ("ffdec_msmpeg4", "v_dec");
+  v_decoder = gst_element_factory_make_or_warn ("ffmpegdecall", "v_dec");
   video_thread = gst_thread_new ("v_decoder_thread");
   videosink = gst_element_factory_make_or_warn ("xvideosink", "v_sink");
   //videosink = gst_element_factory_make_or_warn ("fakesink", "v_sink");
@@ -418,7 +392,6 @@ make_avi_pipeline (const gchar *location)
   return pipeline;
 }
 
-/* An MPEG A/V decoder pipe. */
 static GstElement*
 make_mpeg_pipeline (const gchar *location) 
 {
@@ -434,7 +407,9 @@ make_mpeg_pipeline (const gchar *location)
   g_object_set (G_OBJECT (src), "location", location, NULL);
 
   demux = gst_element_factory_make_or_warn ("mpegdemux", "demux");
-  g_object_set (G_OBJECT (demux), "sync", TRUE, NULL);
+  g_object_set (G_OBJECT (demux), "sync", FALSE, NULL);
+
+  seekable_elements = g_list_prepend (seekable_elements, demux);
 
   gst_bin_add (GST_BIN (pipeline), src);
   gst_bin_add (GST_BIN (pipeline), demux);
@@ -475,9 +450,6 @@ make_mpeg_pipeline (const gchar *location)
 
   setup_dynamic_link (demux, "video_00", gst_element_get_pad (v_decoder, "sink"), video_bin);
 
-  seekable_elements = g_list_prepend (seekable_elements, videosink);
-  seekable_elements = g_list_prepend (seekable_elements, audiosink);
-
   seekable = gst_element_get_pad (v_queue, "src");
   seekable_pads = g_list_prepend (seekable_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, seekable);
@@ -486,7 +458,6 @@ make_mpeg_pipeline (const gchar *location)
   return pipeline;
 }
 
-/* The new version of the MPEG A/V decoder pipe */
 static GstElement*
 make_mpegnt_pipeline (const gchar *location) 
 {
@@ -504,6 +475,8 @@ make_mpegnt_pipeline (const gchar *location)
   demux = gst_element_factory_make_or_warn ("mpegdemux", "demux");
   //g_object_set (G_OBJECT (demux), "sync", TRUE, NULL);
 
+  seekable_elements = g_list_prepend (seekable_elements, demux);
+
   gst_bin_add (GST_BIN (pipeline), src);
   gst_bin_add (GST_BIN (pipeline), demux);
   gst_element_link (src, demux);
@@ -544,12 +517,15 @@ make_mpegnt_pipeline (const gchar *location)
   rate_pads = g_list_prepend (rate_pads, seekable);
   rate_pads = g_list_prepend (rate_pads, gst_element_get_pad (v_decoder, "sink"));
 
-  seekable_elements = g_list_prepend (seekable_elements, audiosink);
-  seekable_elements = g_list_prepend (seekable_elements, videosink);
-
   return pipeline;
 }
 
+static GstElement*
+make_playerbin_pipeline (const gchar *location) 
+{
+  return NULL;
+}
+
 static gchar*
 format_value (GtkScale *scale,
 	      gdouble   value)
@@ -578,7 +554,6 @@ static seek_format seek_formats[] =
 {
   { "tim",  GST_FORMAT_TIME    },
   { "byt",  GST_FORMAT_BYTES   },
-  { "unt",  GST_FORMAT_UNITS   },
   { "buf",  GST_FORMAT_BUFFERS },
   { "def",  GST_FORMAT_DEFAULT },
   { NULL, 0 }, 
@@ -648,7 +623,6 @@ query_durations ()
   }
 }
 
-
 G_GNUC_UNUSED static void
 query_positions ()
 {
@@ -680,7 +654,6 @@ query_positions ()
   }
 }
 
-/* Every 1 s = 1000 ms, update_scale reads the new media time */
 static gboolean
 update_scale (gpointer data) 
 {
@@ -711,7 +684,6 @@ update_scale (gpointer data)
   return TRUE;
 }
 
-/* Make the pipeline play through repeated iteration. */
 static gboolean
 iterate (gpointer data)
 {
@@ -725,9 +697,6 @@ iterate (gpointer data)
   return res;
 }
 
-/* 
-   As soon as the user starts moving the seeking scale, pause playback.
-*/
 static gboolean
 start_seek (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
@@ -737,18 +706,12 @@ start_seek (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
   return FALSE;
 }
 
-/* 
-   After the user has stopped moving the seeking scale, tell the pipeline to seek to the new position.
-*/
 static gboolean
 stop_seek (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
   gint64 real = gtk_range_get_value (GTK_RANGE (widget)) * duration / 100;
   gboolean res;
   GstEvent *s_event;
-
-/* Seeking on sink pads is possible, but deprecated. Use it only
-   if there are no sinks in your application or if element seeking is buggy */
 #ifdef PAD_SEEK
   GList *walk = seekable_pads;
 
@@ -764,11 +727,7 @@ stop_seek (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 
     walk = g_list_next (walk);
   }
-
 #else
-/* Element seeking is the preferred way of seeking. 
-   Tell each sink element (in the A/V file, but videosink and audiosink) to seek.
- */
   GList *walk = seekable_elements;
 
   while (walk) {
@@ -792,7 +751,6 @@ stop_seek (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
   return FALSE;
 }
 
-/* Play button handler */
 static void
 play_cb (GtkButton * button, gpointer data)
 {
@@ -803,7 +761,6 @@ play_cb (GtkButton * button, gpointer data)
   }
 }
 
-/* Pause button handler */
 static void
 pause_cb (GtkButton * button, gpointer data)
 {
@@ -813,7 +770,6 @@ pause_cb (GtkButton * button, gpointer data)
   }
 }
 
-/* Stop button handler */
 static void
 stop_cb (GtkButton * button, gpointer data)
 {
@@ -823,6 +779,43 @@ stop_cb (GtkButton * button, gpointer data)
   }
 }
 
+typedef struct
+{
+  gchar *name;
+  GstElement* (*func) (const gchar *location);
+} Pipeline;
+
+static Pipeline pipelines[] = {
+  { "mp3", 		make_mp3_pipeline 	},
+  { "avi", 		make_avi_pipeline 	},
+  { "mpeg1",  		make_mpeg_pipeline 	},
+  { "mpegparse",  	make_parse_pipeline 	},
+  { "vorbis",  		make_vorbis_pipeline 	}, 
+  { "sid",  		make_sid_pipeline 	},
+  { "flac",  		make_flac_pipeline 	},
+  { "wav",  		make_wav_pipeline 	},
+  { "mod",  		make_mod_pipeline 	},
+  { "dv",  		make_dv_pipeline 	},
+  { "mpeg1nothreads",  	make_mpegnt_pipeline 	},
+  { "playerbin",  	make_playerbin_pipeline },
+  { NULL, NULL},
+};
+
+#define NUM_TYPES	((sizeof (pipelines) / sizeof (Pipeline)) - 1)
+
+static void
+print_usage (int argc, char **argv)
+{
+  gint i;
+
+  g_print ("usage: %s <type> <filename>\n", argv[0]);
+  g_print ("   possible types:\n");
+
+  for (i = 0; i < NUM_TYPES; i++) {
+    g_print ("     %d = %s\n", i, pipelines[i].name);
+  }
+}
+
 int
 main (int argc, char **argv)
 {
@@ -830,41 +823,29 @@ main (int argc, char **argv)
             *play_button, *pause_button, *stop_button, 
 	    *hscale;
   struct poptOption options[] = {
-    {"stats",  's',  POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   &stats,   0,
-           "Show pad stats", NULL},
+    { "stats",  's',  POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   &stats,   0,
+           "Show pad stats", NULL },
     POPT_TABLEEND
   };
+  gint type;
 	        
   gst_init_with_popt_table (&argc, &argv, options);
   gtk_init (&argc, &argv);
 
   if (argc != 3) {
-    g_print ("usage: %s <type 0=mp3 1=avi 2=mpeg1 3=mpegparse 4=vorbis 5=sid 6=flac 7=wav 8=mod 9=dv 10=mpeg1nothreads> <filename>\n", argv[0]);
+    print_usage (argc, argv);
     exit (-1);
   }
 
-  if (atoi (argv[1]) == 0) 
-    pipeline = make_mp3_pipeline (argv[2]);
-  else if (atoi (argv[1]) == 1) 
-    pipeline = make_avi_pipeline (argv[2]);
-  else if (atoi (argv[1]) == 2) 
-    pipeline = make_mpeg_pipeline (argv[2]);
-  else if (atoi (argv[1]) == 3)
-    pipeline = make_parse_pipeline (argv[2]);
-  else if (atoi (argv[1]) == 4) 
-    pipeline = make_vorbis_pipeline (argv[2]);
-  else if (atoi (argv[1]) == 5) 
-    pipeline = make_sid_pipeline (argv[2]);
-  else if (atoi (argv[1]) == 6) 
-    pipeline = make_flac_pipeline (argv[2]);
-  else if (atoi (argv[1]) == 7) 
-    pipeline = make_wav_pipeline (argv[2]);
-  else if (atoi (argv[1]) == 8) 
-    pipeline = make_mod_pipeline (argv[2]);
-  else if (atoi (argv[1]) == 9) 
-    pipeline = make_dv_pipeline (argv[2]);
-  else if (atoi (argv[1]) == 10) 
-    pipeline = make_mpegnt_pipeline (argv[2]);
+  type = atoi (argv[1]);
+
+  if (type < 0 || type >= NUM_TYPES) {
+    print_usage (argc, argv);
+    exit (-1);
+  }
+
+  pipeline = pipelines[type].func (argv[2]);
+  g_assert (pipeline);
 
   /* initialize gui elements ... */
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -912,8 +893,6 @@ main (int argc, char **argv)
   gst_element_set_state (pipeline, GST_STATE_NULL);
 
   //gst_object_unref (GST_OBJECT (pipeline));
-  gst_buffer_print_stats();
-  gst_event_print_stats();
 
   //g_mem_chunk_info();
 
