@@ -42,7 +42,7 @@ print_caps (GstCaps *caps)
       GstPropsType type;
 
       name = gst_props_entry_get_name (entry);
-      type = gst_props_entry_get_type (entry);
+      type = gst_props_entry_get_props_type (entry);
       switch (type) {
 	case GST_PROPS_STRING_TYPE:
           gst_props_entry_get_string (entry, &str_val);
@@ -316,8 +316,5 @@ main (int argc, char **argv)
   /* stop probe */
   gst_element_set_state (pipeline, GST_STATE_NULL);
 
-  gst_buffer_print_stats();
-  gst_event_print_stats();
-
   return 0;
 }
