@@ -17,6 +17,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 #include <sys/soundcard.h>
 
@@ -46,7 +49,7 @@ enum {
 
 enum {
   ARG_0,
-  ARG_METADATA
+	ARG_METADATA
 };
 
 static void 		gst_flacdec_class_init		(FlacDecClass *klass);
@@ -65,14 +68,14 @@ static gboolean		gst_flacdec_src_query 		(GstPad *pad, GstQueryType type,
 static const GstEventMask* 
 			gst_flacdec_get_src_event_masks (GstPad *pad);
 static gboolean 	gst_flacdec_src_event 		(GstPad *pad, GstEvent *event);
-static void             gst_flacdec_get_property        (GObject *object, 
-                                                         guint prop_id, 
-                                                         GValue *value, 
-                                                         GParamSpec *pspec);
-static void             gst_flacdec_set_property        (GObject *object, 
-                                                         guint prop_id, 
-                                                         const GValue *value, 
-                                                         GParamSpec *pspec);
+static void 	gst_flacdec_get_property 	(GObject *object, 
+		            			 guint prop_id, 
+						 GValue *value, 
+						 GParamSpec *pspec);
+static void 	gst_flacdec_set_property 	(GObject *object, 
+		            			 guint prop_id, 
+						 const GValue *value, 
+						 GParamSpec *pspec);
 
 static FLAC__SeekableStreamDecoderReadStatus 	
 			gst_flacdec_read 		(const FLAC__SeekableStreamDecoder *decoder, 
@@ -129,18 +132,18 @@ static void
 gst_flacdec_class_init (FlacDecClass *klass) 
 {
   GstElementClass *gstelement_class;
-  GObjectClass *gobject_class;
+	GObjectClass *gobject_class;
 
   gstelement_class = (GstElementClass*)klass;
-  gobject_class = (GObjectClass*) klass;
+	gobject_class = (GObjectClass*) klass;
 
   parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
 	
-  g_object_class_install_property (gobject_class, ARG_METADATA,
-  g_param_spec_boxed ("metadata", "Metadata", "(logical) Stream metadata",
-                      GST_TYPE_CAPS, G_PARAM_READABLE));
+	g_object_class_install_property (gobject_class, ARG_METADATA,
+    g_param_spec_boxed ("metadata", "Metadata", "(logical) Stream metadata",
+                         GST_TYPE_CAPS, G_PARAM_READABLE));
 
-  gobject_class->get_property = gst_flacdec_get_property;
+	gobject_class->get_property = gst_flacdec_get_property;
   gobject_class->set_property = gst_flacdec_set_property;
 	
   gstelement_class->change_state = gst_flacdec_change_state;
@@ -168,7 +171,7 @@ gst_flacdec_init (FlacDec *flacdec)
   flacdec->init = TRUE;
   flacdec->eos = FALSE;
   flacdec->seek_pending = FALSE;
-  flacdec->metadata = NULL;
+	flacdec->metadata = NULL;
 
   FLAC__seekable_stream_decoder_set_read_callback (flacdec->decoder, gst_flacdec_read);
   FLAC__seekable_stream_decoder_set_seek_callback (flacdec->decoder, gst_flacdec_seek);
@@ -211,7 +214,7 @@ gst_flacdec_update_metadata (FlacDec *flacdec, const FLAC__StreamMetadata *metad
   
   number_of_comments = metadata->data.vorbis_comment.num_comments;
   value = NULL;
-  GST_DEBUG (GST_CAT_CAPS, "%d tag(s) found",  number_of_comments);
+  GST_DEBUG ("%d tag(s) found",  number_of_comments);
   for (cursor = 0; cursor < number_of_comments; cursor++)
   {			
     str_ptr = metadata->data.vorbis_comment.comments[cursor].entry;
@@ -225,7 +228,7 @@ gst_flacdec_update_metadata (FlacDec *flacdec, const FLAC__StreamMetadata *metad
       entry = gst_props_entry_new (name, GST_PROPS_STRING_TYPE, value);
       gst_props_add_entry (props, (GstPropsEntry *) entry);
 
-      GST_DEBUG (GST_CAT_CAPS, "%s : %s", name, value);
+      GST_DEBUG ("%s : %s", name, value);
 
       g_free (name);
       g_free (value);
@@ -240,6 +243,7 @@ gst_flacdec_update_metadata (FlacDec *flacdec, const FLAC__StreamMetadata *metad
   return TRUE;
 }
 
+
 static void 
 gst_flacdec_metadata_callback (const FLAC__SeekableStreamDecoder *decoder,
 			       const FLAC__StreamMetadata *metadata, void *client_data)
@@ -285,7 +289,7 @@ gst_flacdec_error_callback (const FLAC__SeekableStreamDecoder *decoder,
       break;
   }
 
-  GST_DEBUG (0, error);
+  GST_DEBUG (error);
 				  
   gst_element_error (GST_ELEMENT (flacdec), error);
 }
@@ -298,7 +302,7 @@ gst_flacdec_seek (const FLAC__SeekableStreamDecoder *decoder,
 
   flacdec = GST_FLACDEC (client_data);
 
-  GST_DEBUG (0, "seek %" G_GINT64_FORMAT, position);
+  GST_DEBUG ("seek %" G_GINT64_FORMAT, position);
   if (!gst_bytestream_seek (flacdec->bs, position, GST_SEEK_METHOD_SET)) {
     return FLAC__SEEKABLE_STREAM_DECODER_SEEK_STATUS_ERROR;
   }
@@ -317,12 +321,11 @@ gst_flacdec_tell (const FLAC__SeekableStreamDecoder *decoder,
   if (*position == -1)
     return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_ERROR;
 
-  GST_DEBUG (0, "tell %" G_GINT64_FORMAT, *position);
+  GST_DEBUG ("tell %" G_GINT64_FORMAT, *position);
 
   return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_OK;
 }
 
-
 static FLAC__SeekableStreamDecoderLengthStatus 	
 gst_flacdec_length (const FLAC__SeekableStreamDecoder *decoder, 
 		    FLAC__uint64 *length, void *client_data)
@@ -335,7 +338,7 @@ gst_flacdec_length (const FLAC__SeekableStreamDecoder *decoder,
   if (*length == -1)
     return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_ERROR;
 
-  GST_DEBUG (0, "length %" G_GINT64_FORMAT, *length);
+  GST_DEBUG ("length %" G_GINT64_FORMAT, *length);
 
   return FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_OK;
 }
@@ -347,7 +350,7 @@ gst_flacdec_eof (const FLAC__SeekableStreamDecoder *decoder,
   FlacDec *flacdec;
 
   flacdec = GST_FLACDEC (client_data);
-  GST_DEBUG (0, "eof %d", flacdec->eos);
+  GST_DEBUG ("eof %d", flacdec->eos);
 
   return flacdec->eos;
 }
@@ -375,7 +378,7 @@ gst_flacdec_read (const FLAC__SeekableStreamDecoder *decoder,
 
       switch (GST_EVENT_TYPE (event)) {
         case GST_EVENT_EOS:
-          GST_DEBUG (0, "eos");
+          GST_DEBUG ("eos");
           flacdec->eos = TRUE; 
 	  gst_event_unref (event);
           if (avail == 0) {
@@ -383,7 +386,7 @@ gst_flacdec_read (const FLAC__SeekableStreamDecoder *decoder,
           }
           break;
         case GST_EVENT_DISCONTINUOUS:
-          GST_DEBUG (0, "discont");
+          GST_DEBUG ("discont");
 
 	  /* we are not yet sending the discont, we'll do that in the next write operation */
 	  flacdec->need_discont = TRUE;
@@ -428,26 +431,28 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder *decoder,
     GstEvent *discont;
 
     flacdec->need_discont = FALSE;
+    
+    if (!GST_PAD_CAPS (flacdec->srcpad)) {
+      if (flacdec->seek_pending) {
+        flacdec->total_samples = flacdec->seek_value;
+      }
 
-    if (flacdec->seek_pending) {
-      flacdec->total_samples = flacdec->seek_value;
-    }
+      if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
+        GST_DEBUG ("send discont");
 
-    if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
-      GST_DEBUG (0, "send discont");
-
-      format = GST_FORMAT_TIME;
-      gst_pad_convert (flacdec->srcpad, GST_FORMAT_UNITS,
-                       flacdec->total_samples, &format, &time);
-      format = GST_FORMAT_BYTES;
-      gst_pad_convert (flacdec->srcpad, GST_FORMAT_UNITS,
-                       flacdec->total_samples, &format, &bytes);
-      discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time,
-		                         GST_FORMAT_BYTES, bytes,
-		                         GST_FORMAT_UNITS, flacdec->total_samples, 
+        format = GST_FORMAT_TIME;
+        gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
+                         flacdec->total_samples, &format, &time);
+        format = GST_FORMAT_BYTES;
+        gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
+                         flacdec->total_samples, &format, &bytes);
+        discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time,
+	  	                             GST_FORMAT_BYTES, bytes,
+		                             GST_FORMAT_DEFAULT, flacdec->total_samples, 
 					 NULL);
 	  
-      gst_pad_push (flacdec->srcpad, GST_BUFFER (discont));
+        gst_pad_push (flacdec->srcpad, GST_DATA (discont));
+      }
     }
   }
   
@@ -455,9 +460,7 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder *decoder,
     gst_pad_try_set_caps (flacdec->srcpad,
 		    GST_CAPS_NEW (
 		      "flac_caps",
-		      "audio/raw",
-		        "format", 	GST_PROPS_STRING ("int"),
-                         "law",         GST_PROPS_INT (0),
+		      "audio/x-raw-int",
                          "endianness",  GST_PROPS_INT (G_BYTE_ORDER),
                          "signed",      GST_PROPS_BOOLEAN (TRUE),
                          "width",       GST_PROPS_INT (depth),
@@ -499,7 +502,7 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder *decoder,
       g_warning ("flacdec: invalid depth %d found\n", depth);
       return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
     }
-    gst_pad_push (flacdec->srcpad, outbuf);
+    gst_pad_push (flacdec->srcpad, GST_DATA (outbuf));
   }
   flacdec->total_samples += samples;
 
@@ -514,52 +517,53 @@ gst_flacdec_loop (GstElement *element)
 
   flacdec = GST_FLACDEC (element);
 
-  GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: entering loop");
+  GST_DEBUG ("flacdec: entering loop");
   if (flacdec->init) {
-    GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: initializing decoder");
+    GST_DEBUG ("flacdec: initializing decoder");
     FLAC__seekable_stream_decoder_init (flacdec->decoder);
     /* FLAC__seekable_stream_decoder_process_metadata (flacdec->decoder); */
     flacdec->init = FALSE;
   }
 
   if (flacdec->seek_pending) {
-    GST_DEBUG (GST_CAT_EVENT, "perform seek to sample %" G_GINT64_FORMAT, 
+    GST_DEBUG ("perform seek to sample %" G_GINT64_FORMAT, 
 		              flacdec->seek_value);
 
     if (FLAC__seekable_stream_decoder_seek_absolute (flacdec->decoder, 
 			                             flacdec->seek_value)) 
     {
       flacdec->total_samples = flacdec->seek_value;
-      GST_DEBUG (GST_CAT_EVENT, "seek done");
+      GST_DEBUG ("seek done");
     }
     else {
-      GST_DEBUG (GST_CAT_EVENT, "seek failed");
+      GST_DEBUG ("seek failed");
     }
     flacdec->seek_pending = FALSE;
   }
 
-  GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: processing single");
+  GST_DEBUG ("flacdec: processing single");
   res = FLAC__seekable_stream_decoder_process_single (flacdec->decoder);
-  GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: checking for EOS");
+  GST_DEBUG ("flacdec: checking for EOS");
   if (FLAC__seekable_stream_decoder_get_state (flacdec->decoder) == 
 		  FLAC__SEEKABLE_STREAM_DECODER_END_OF_STREAM) 
   {
     GstEvent *event;
 
-    GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: sending EOS event");
-    FLAC__seekable_stream_decoder_reset(flacdec->decoder);
+    GST_DEBUG ("flacdec: sending EOS event");
+    FLAC__seekable_stream_decoder_finish(flacdec->decoder);
+    flacdec->init = TRUE;
 
     if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
       event = gst_event_new (GST_EVENT_EOS);
-      gst_pad_push (flacdec->srcpad, GST_BUFFER (event));
+      gst_pad_push (flacdec->srcpad, GST_DATA (event));
     }
     gst_element_set_eos (element);
   }
-   GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: _loop end");
+  GST_DEBUG ("flacdec: _loop end");
 }
 
 GST_PAD_FORMATS_FUNCTION (gst_flacdec_get_src_formats,
-  GST_FORMAT_UNITS,
+  GST_FORMAT_DEFAULT,
   GST_FORMAT_BYTES,
   GST_FORMAT_TIME
 )
@@ -578,13 +582,11 @@ gst_flacdec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
           if (bytes_per_sample == 0)
             return FALSE;
           *dest_value = src_value / bytes_per_sample;
           break;
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
 	{
           gint byterate = bytes_per_sample * flacdec->frequency;
@@ -598,13 +600,11 @@ gst_flacdec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
           res = FALSE;
       }
       break;
-    case GST_FORMAT_UNITS:
+    case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
 	  *dest_value = src_value * bytes_per_sample;
           break;
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
 	  if (flacdec->frequency == 0)
 	    return FALSE;
@@ -616,11 +616,9 @@ gst_flacdec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
           scale = bytes_per_sample;
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  *dest_value = src_value * scale * flacdec->frequency / GST_SECOND;
           break;
         default:
@@ -656,14 +654,14 @@ gst_flacdec_src_query (GstPad *pad, GstQueryType type,
         samples = flacdec->stream_samples;
 
       gst_pad_convert (flacdec->srcpad, 
-		       GST_FORMAT_UNITS, 
+		       GST_FORMAT_DEFAULT, 
 		       samples, 
 		       format, value);
       break;
     }
     case GST_QUERY_POSITION:
       gst_pad_convert (flacdec->srcpad, 
-		       GST_FORMAT_UNITS, 
+		       GST_FORMAT_DEFAULT, 
 		       flacdec->total_samples, 
 		       format, value);
       break;
@@ -688,7 +686,7 @@ gst_flacdec_src_event (GstPad *pad, GstEvent *event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
-      format = GST_FORMAT_UNITS;
+      format = GST_FORMAT_DEFAULT;
 
       if (gst_pad_convert (flacdec->srcpad, 
 			   GST_EVENT_SEEK_FORMAT (event), 
@@ -719,7 +717,7 @@ gst_flacdec_change_state (GstElement *element)
       flacdec->total_samples = 0;
       flacdec->init = TRUE;
       flacdec->eos = FALSE;
-      flacdec->metadata = NULL;
+      FLAC__seekable_stream_decoder_reset (flacdec->decoder);
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
       flacdec->eos = FALSE;
@@ -741,7 +739,7 @@ gst_flacdec_change_state (GstElement *element)
 
 static void
 gst_flacdec_set_property (GObject *object, guint prop_id, 
-                          const GValue *value, GParamSpec *pspec)
+		             const GValue *value, GParamSpec *pspec)
 {
   FlacDec *flacdec;
 	      
@@ -757,7 +755,7 @@ gst_flacdec_set_property (GObject *object, guint prop_id,
 
 static void 
 gst_flacdec_get_property (GObject *object, guint prop_id, 
-                          GValue *value, GParamSpec *pspec)
+		             GValue *value, GParamSpec *pspec)
 {
   FlacDec *flacdec;
 	      
