@@ -18,6 +18,9 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdlib.h>
 #include <string.h>
 
@@ -59,7 +62,6 @@ enum {
   ARG_MIN_RESIDUAL_PARTITION_ORDER,
   ARG_MAX_RESIDUAL_PARTITION_ORDER,
   ARG_RICE_PARAMETER_SEARCH_DIST,
-  ARG_METADATA
 };
 
 static void		gst_flacenc_init		(FlacEnc *flacenc);
@@ -68,7 +70,7 @@ static void 		gst_flacenc_dispose 		(GObject *object);
 
 static GstPadLinkReturn
 			gst_flacenc_sinkconnect 	(GstPad *pad, GstCaps *caps);
-static void		gst_flacenc_chain		(GstPad *pad, GstBuffer *buf);
+static void		gst_flacenc_chain		(GstPad *pad, GstData *_data);
 
 static gboolean 	gst_flacenc_update_quality 	(FlacEnc *flacenc, gint quality);
 static void     	gst_flacenc_set_property        (GObject *object, guint prop_id, 
@@ -257,9 +259,6 @@ gst_flacenc_class_init (FlacEncClass *klass)
 		         "parameters, use best",
                        0, FLAC__MAX_RICE_PARTITION_ORDER, 
                        flacenc_params[DEFAULT_QUALITY].rice_parameter_search_dist, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_METADATA,
-    g_param_spec_boxed ("metadata", "Metadata", "Metadata to add to the stream,",
-                        GST_TYPE_CAPS, G_PARAM_READWRITE));		       
 
   gstelement_class->change_state = gst_flacenc_change_state;
 }
@@ -279,16 +278,31 @@ gst_flacenc_init (FlacEnc *flacenc)
 
   flacenc->encoder = FLAC__seekable_stream_encoder_new();
 
+  FLAC__seekable_stream_encoder_set_write_callback (flacenc->encoder, 
+		                        gst_flacenc_write_callback);
+  FLAC__seekable_stream_encoder_set_seek_callback (flacenc->encoder, 
+		                        gst_flacenc_seek_callback);
+  					
+  FLAC__seekable_stream_encoder_set_client_data (flacenc->encoder, 
+		                        flacenc);
+
   flacenc->metadata = GST_CAPS_NEW (
                   "flacenc_metadata",
                   "application/x-gst-metadata",
-		    "comment",  GST_PROPS_STRING ("Track encoded with GStreamer"),
-                    "date",     GST_PROPS_STRING (""),
-		    "tracknum", GST_PROPS_STRING (""),
-		    "title",    GST_PROPS_STRING (""),
-		    "artist",   GST_PROPS_STRING (""),
-		    "album",    GST_PROPS_STRING (""),
-		    "genre",    GST_PROPS_STRING ("")
+                    "DESCRIPTION",      GST_PROPS_STRING ("Track encoded with GStreamer"),
+                    "DATE",             GST_PROPS_STRING (""),
+                    "TRACKNUMBER",      GST_PROPS_STRING (""),
+                    "TITLE",            GST_PROPS_STRING (""),
+                    "ARTIST",           GST_PROPS_STRING (""),
+                    "ALBUM",            GST_PROPS_STRING (""),
+                    "GENRE",            GST_PROPS_STRING (""),
+                    "VERSION",          GST_PROPS_STRING (""),
+                    "COPYRIGHT",        GST_PROPS_STRING (""),
+                    "LICENSE",          GST_PROPS_STRING (""),
+                    "ORGANISATION",     GST_PROPS_STRING (""),
+                    "LOCATION",         GST_PROPS_STRING (""),
+                    "CONTACT",          GST_PROPS_STRING (""),
+                    "ISRC",             GST_PROPS_STRING ("")
                   );
 		  
   flacenc->negotiated = FALSE;
@@ -311,6 +325,7 @@ gst_flacenc_dispose (GObject *object)
 static GstPadLinkReturn
 gst_flacenc_sinkconnect (GstPad *pad, GstCaps *caps)
 {
+  GstPadLinkReturn ret;
   FlacEnc *flacenc;
 
   flacenc = GST_FLACENC (gst_pad_get_parent (pad));
@@ -322,6 +337,15 @@ gst_flacenc_sinkconnect (GstPad *pad, GstCaps *caps)
   gst_caps_get_int (caps, "depth", &flacenc->depth);
   gst_caps_get_int (caps, "rate", &flacenc->sample_rate);
   
+  caps = GST_CAPS_NEW ("flacenc_srccaps",
+                       "audio/x-flac",
+                         "channels", GST_PROPS_INT (flacenc->channels),
+                         "rate", GST_PROPS_INT (flacenc->sample_rate));
+  ret = gst_pad_try_set_caps (flacenc->srcpad, caps);
+  if (ret <= 0) {
+    return ret;
+  }
+
   FLAC__seekable_stream_encoder_set_bits_per_sample (flacenc->encoder, 
 		  			    flacenc->depth);
   FLAC__seekable_stream_encoder_set_sample_rate (flacenc->encoder, 
@@ -331,7 +355,7 @@ gst_flacenc_sinkconnect (GstPad *pad, GstCaps *caps)
 
   flacenc->negotiated = TRUE;
 
-  return GST_PAD_LINK_OK;
+  return ret;
 }
 
 static gboolean
@@ -386,7 +410,7 @@ GstEvent *event;
   event = gst_event_new_seek ((GstSeekType)(int)(GST_FORMAT_BYTES | GST_SEEK_METHOD_SET), absolute_byte_offset); 
 
   if (event)
-    gst_pad_push (flacenc->srcpad, GST_BUFFER (event));
+    gst_pad_push (flacenc->srcpad, GST_DATA (event));
  
   return  FLAC__STREAM_ENCODER_OK;
 }
@@ -415,7 +439,7 @@ gst_flacenc_write_callback (const FLAC__SeekableStreamEncoder *encoder,
     flacenc->first = FALSE;
   }
 
-  gst_pad_push (flacenc->srcpad, outbuf);
+  gst_pad_push (flacenc->srcpad, GST_DATA (outbuf));
 
   return FLAC__STREAM_ENCODER_OK;
 }
@@ -423,40 +447,35 @@ gst_flacenc_write_callback (const FLAC__SeekableStreamEncoder *encoder,
 static void
 gst_flacenc_set_metadata (FlacEnc *flacenc, GstCaps *caps)
 {
-  GList *props;
-  GstPropsEntry *prop;
+  guint indice;
   guint comments;
+  const gchar *meta_types[] = { "TITLE", "VERSION", "ALBUM", "TRACKNUMBER",
+                                "ARTIST", "PERFORMER", "COPYRIGHT", "LICENSE",
+                                "ORGANISATION", "DESCRIPTION", "GENRE", "DATE",
+                                "LOCATION", "CONTACT", "ISRC", NULL };
 
   g_return_if_fail (flacenc != NULL);
 
   flacenc->meta = g_malloc (sizeof (FLAC__StreamMetadata **));
 
   flacenc->meta[0] = FLAC__metadata_object_new (FLAC__METADATA_TYPE_VORBIS_COMMENT);
-
-  comments = 0;
-  props = gst_caps_get_props (caps)->properties;
-  while (props) {
-    prop = (GstPropsEntry*)(props->data);
-    props = g_list_next(props);
-
-    if (gst_props_entry_get_type (prop) == GST_PROPS_STRING_TYPE) {
-      const gchar *name = gst_props_entry_get_name (prop);
-      const gchar *value;
+  
+  for ( indice = 0, comments=0; meta_types[indice] != NULL; indice++) {
+    if (gst_caps_has_property(caps, meta_types[indice])) {
+      const gchar *entry;
       gchar *data;
       FLAC__StreamMetadata_VorbisComment_Entry commment_entry;
+      
+      gst_caps_get_string(caps, (gchar *)meta_types[indice], &entry);
 
-      gst_props_entry_get_string (prop, &value);
-
-      if (!value || strlen (value) == 0)
-        continue;
-
-      data = g_strdup_printf("%s=%s", name, value);
-
-      commment_entry.length = GUINT32_TO_LE (strlen(data));
-      commment_entry.entry  = g_convert (data, commment_entry.length, "UTF8", "ISO-8859-1", NULL, NULL, NULL);
+      if (!strcmp (entry, ""))
+	continue;
+   
+      data = g_strdup_printf("%s=%s", meta_types[indice], entry);
+      commment_entry.length = GUINT32_TO_LE (strlen(entry) + strlen(meta_types[indice]) + 1);
+      commment_entry.entry  =  g_convert (data, commment_entry.length, "UTF8", "ISO-8859-1", NULL, NULL, NULL);
 
       FLAC__metadata_object_vorbiscomment_insert_comment (flacenc->meta[0], comments++, commment_entry, TRUE);
-      g_free (data);
     }
   }
 
@@ -466,8 +485,9 @@ gst_flacenc_set_metadata (FlacEnc *flacenc, GstCaps *caps)
 
 
 static void
-gst_flacenc_chain (GstPad *pad, GstBuffer *buf)
+gst_flacenc_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   FlacEnc *flacenc;
   FLAC__int32 *data;
   gulong insize;
@@ -481,15 +501,10 @@ gst_flacenc_chain (GstPad *pad, GstBuffer *buf)
 
   if (GST_IS_EVENT (buf)) {
     GstEvent *event = GST_EVENT (buf);
-    GstEvent *eos_event;
 
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_EOS:
-        FLAC__seekable_stream_encoder_finish (flacenc->encoder);
-        eos_event = gst_event_new (GST_EVENT_EOS);
-        gst_pad_push (flacenc->srcpad, GST_BUFFER (eos_event));
-        gst_element_set_eos (GST_ELEMENT (flacenc));
-        break;
+	FLAC__seekable_stream_encoder_finish(flacenc->encoder);
       default:
 	gst_pad_event_default (pad, event);
         break;
@@ -513,14 +528,6 @@ gst_flacenc_chain (GstPad *pad, GstBuffer *buf)
   {
     FLAC__SeekableStreamEncoderState state;
 
-    FLAC__seekable_stream_encoder_set_write_callback (flacenc->encoder, 
-		                        gst_flacenc_write_callback);
-    FLAC__seekable_stream_encoder_set_seek_callback (flacenc->encoder, 
-		                        gst_flacenc_seek_callback);
-  					
-    FLAC__seekable_stream_encoder_set_client_data (flacenc->encoder, 
-		                        flacenc);
-
     gst_flacenc_set_metadata (flacenc, flacenc->metadata);
     state = FLAC__seekable_stream_encoder_init (flacenc->encoder);
     if (state != FLAC__STREAM_ENCODER_OK) {
@@ -622,9 +629,6 @@ gst_flacenc_set_property (GObject *object, guint prop_id,
     case ARG_RICE_PARAMETER_SEARCH_DIST:
       FLAC__seekable_stream_encoder_set_rice_parameter_search_dist (this->encoder, 
 		                                 g_value_get_uint (value));
-      break; 
-    case ARG_METADATA:
-      this->metadata = g_value_get_boxed (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -692,9 +696,6 @@ gst_flacenc_get_property (GObject *object, guint prop_id,
       g_value_set_uint (value, 
 	      FLAC__seekable_stream_encoder_get_rice_parameter_search_dist (this->encoder));
       break;
-    case ARG_METADATA:
-      g_value_set_static_boxed (value, this->metadata);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
