@@ -17,7 +17,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gst/gst.h>
+#include <gst/video/video.h>
 
 #include "gstcolorspace.h"
 #include "yuv2rgb.h"
@@ -26,7 +30,7 @@
 
 static GstElementDetails colorspace_details = {
   "Colorspace converter",
-  "Filter/Video",
+  "Filter/Convert",
   "LGPL",
   "Converts video from one colorspace to another using libhermes",
   VERSION,
@@ -47,42 +51,6 @@ enum {
   ARG_DEST,
 };
 
-GST_PAD_TEMPLATE_FACTORY (colorspace_src_template_factory,
-  "src",
-  GST_PAD_SRC,
-  GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "colorspace_src",
-    "video/raw",
-      "format",		GST_PROPS_LIST (
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("YV12")),
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("YUY2")),
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("RGB "))
-	                ),
-      "width", 		GST_PROPS_INT_RANGE (0, G_MAXINT),
-      "height",		GST_PROPS_INT_RANGE (0, G_MAXINT) 
-  )
-)
-
-GST_PAD_TEMPLATE_FACTORY (colorspace_sink_template_factory,
-  "sink",
-  GST_PAD_SINK,
-  GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "colorspace_sink",
-    "video/raw",
-      "format",		GST_PROPS_LIST (
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("YV12")),
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("YUY2")),
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("RGB "))
-	                ),
-      "width", 		GST_PROPS_INT_RANGE (0, G_MAXINT),
-      "height",		GST_PROPS_INT_RANGE (0, G_MAXINT) 
-  )
-)
-
 static void		gst_colorspace_class_init		(GstColorspaceClass *klass);
 static void		gst_colorspace_init			(GstColorspace *space);
 
@@ -97,7 +65,7 @@ static GstPadLinkReturn
 			gst_colorspace_srcconnect 		(GstPad *pad, GstCaps *caps);
 static GstPadLinkReturn
 			gst_colorspace_srcconnect_func 		(GstPad *pad, GstCaps *caps, gboolean newcaps);
-static void		gst_colorspace_chain			(GstPad *pad, GstBuffer *buf);
+static void		gst_colorspace_chain			(GstPad *pad, GstData *_data);
 static GstElementStateReturn
 			gst_colorspace_change_state 		(GstElement *element);
 
@@ -107,6 +75,7 @@ extern void 	gst_colorspace_rgb32_to_i420	(unsigned char *src, unsigned char *de
 extern void 	gst_colorspace_rgb32_to_yv12	(unsigned char *src, unsigned char *dest, 
 						 guint width, guint height);
 
+static GstPadTemplate *srctempl, *sinktempl;
 static GstElementClass *parent_class = NULL;
 /*static guint gst_colorspace_signals[LAST_SIGNAL] = { 0 }; */
 
@@ -131,10 +100,20 @@ colorspace_setup_converter (GstColorspace *space, GstCaps *from_caps, GstCaps *t
   g_return_val_if_fail (to_caps != NULL, FALSE);
   g_return_val_if_fail (from_caps != NULL, FALSE);
 
-  gst_caps_get_fourcc_int (from_caps, "format", &from_space);
-  gst_caps_get_fourcc_int (to_caps,   "format", &to_space);
+  if (gst_caps_has_property (from_caps, "format"))
+    gst_caps_get_fourcc_int (from_caps, "format", &from_space);
+  else
+    from_space = GST_MAKE_FOURCC ('R','G','B',' ');
 
-  GST_INFO (GST_CAT_NEGOTIATION, "set up converter for %08x to %08x", from_space, to_space);
+  if (gst_caps_has_property (to_caps, "format"))
+    gst_caps_get_fourcc_int (to_caps, "format", &to_space);
+  else
+    to_space = GST_MAKE_FOURCC ('R','G','B',' ');
+
+  GST_INFO ("set up converter for "  GST_FOURCC_FORMAT
+	    " (%08x) to " GST_FOURCC_FORMAT " (%08x)",
+	    GST_FOURCC_ARGS (from_space), from_space,
+	    GST_FOURCC_ARGS (to_space), to_space);
 
   switch (from_space) {
     case GST_MAKE_FOURCC ('R','G','B',' '):
@@ -159,10 +138,10 @@ colorspace_setup_converter (GstColorspace *space, GstCaps *from_caps, GstCaps *t
 	  space->source.indexed = 0;
 	  space->source.has_colorkey = 0;
 
-	  GST_INFO (GST_CAT_PLUGIN_INFO, "source red mask   %08x", space->source.r);
-	  GST_INFO (GST_CAT_PLUGIN_INFO, "source green mask %08x", space->source.g);
-	  GST_INFO (GST_CAT_PLUGIN_INFO, "source blue mask  %08x", space->source.b);
-	  GST_INFO (GST_CAT_PLUGIN_INFO, "source bpp        %08x", space->srcbpp);
+	  GST_INFO ( "source red mask   %08x", space->source.r);
+	  GST_INFO ( "source green mask %08x", space->source.g);
+	  GST_INFO ( "source blue mask  %08x", space->source.b);
+	  GST_INFO ( "source bpp        %08x", space->srcbpp);
 
 	  gst_caps_get_int (to_caps, "red_mask",   &space->dest.r);
 	  gst_caps_get_int (to_caps, "green_mask", &space->dest.g);
@@ -172,16 +151,16 @@ colorspace_setup_converter (GstColorspace *space, GstCaps *from_caps, GstCaps *t
 	  space->dest.indexed = 0;
 	  space->dest.has_colorkey = 0;
 
-	  GST_INFO (GST_CAT_PLUGIN_INFO, "dest red mask   %08x", space->dest.r);
-	  GST_INFO (GST_CAT_PLUGIN_INFO, "dest green mask %08x", space->dest.g);
-	  GST_INFO (GST_CAT_PLUGIN_INFO, "dest blue mask  %08x", space->dest.b);
-	  GST_INFO (GST_CAT_PLUGIN_INFO, "dest bpp        %08x", space->destbpp);
+	  GST_INFO ( "dest red mask   %08x", space->dest.r);
+	  GST_INFO ( "dest green mask %08x", space->dest.g);
+	  GST_INFO ( "dest blue mask  %08x", space->dest.b);
+	  GST_INFO ( "dest bpp        %08x", space->destbpp);
 
 	  if (!Hermes_ConverterRequest (space->h_handle, &space->source, &space->dest)) {
 	    g_warning ("Hermes: could not get converter\n");
 	    return FALSE;
 	  }
-	  GST_INFO (GST_CAT_PLUGIN_INFO, "converter set up");
+	  GST_INFO ( "converter set up");
           space->type = GST_COLORSPACE_HERMES;
 	  return TRUE;
 	}
@@ -202,7 +181,7 @@ colorspace_setup_converter (GstColorspace *space, GstCaps *from_caps, GstCaps *t
 	    return TRUE;
 	  }
         case GST_MAKE_FOURCC ('Y','U','Y','2'):
-          GST_INFO (GST_CAT_NEGOTIATION, "colorspace: RGB to YUV with bpp %d not implemented!!", from_bpp);
+          GST_INFO ( "colorspace: RGB to YUV with bpp %d not implemented!!", from_bpp);
 	  return FALSE;
       }
       break;
@@ -210,7 +189,7 @@ colorspace_setup_converter (GstColorspace *space, GstCaps *from_caps, GstCaps *t
     case GST_MAKE_FOURCC ('I','4','2','0'):
       switch (to_space) {
         case GST_MAKE_FOURCC ('R','G','B',' '):
-          GST_INFO (GST_CAT_NEGOTIATION, "colorspace: YUV to RGB");
+          GST_INFO ( "colorspace: YUV to RGB");
 
 	  gst_caps_get_int (to_caps, "bpp", &space->destbpp);
 	  space->converter = gst_colorspace_yuv2rgb_get_converter (from_caps, to_caps);
@@ -238,14 +217,14 @@ colorspace_setup_converter (GstColorspace *space, GstCaps *from_caps, GstCaps *t
 	  space->destbpp = 16;
 	  return TRUE;
         case GST_MAKE_FOURCC ('R','G','B',' '):
-          GST_INFO (GST_CAT_NEGOTIATION, "colorspace: YUY2 to RGB not implemented!!");
+          GST_INFO ( "colorspace: YUY2 to RGB not implemented!!");
 	  return FALSE;
       }
       break;
     case GST_MAKE_FOURCC ('Y','V','1','2'):
       switch (to_space) {
         case GST_MAKE_FOURCC ('R','G','B',' '):
-          GST_INFO (GST_CAT_NEGOTIATION, "colorspace: YV12 to RGB");
+          GST_INFO ( "colorspace: YV12 to RGB");
 
 	  gst_caps_get_int (to_caps, "bpp", &space->destbpp);
 	  space->converter = gst_colorspace_yuv2rgb_get_converter (from_caps, to_caps);
@@ -305,8 +284,9 @@ gst_colorspace_sinkconnect (GstPad *pad, GstCaps *caps)
 
   gst_caps_get_int (caps, "width", &space->width);
   gst_caps_get_int (caps, "height", &space->height);
+  gst_caps_get_float (caps, "framerate", &space->fps);
 
-  GST_INFO (GST_CAT_PROPERTIES, "size: %dx%d", space->width, space->height);
+  GST_INFO ( "size: %dx%d", space->width, space->height);
 
   gst_caps_replace_sink (&space->sinkcaps, caps);
 
@@ -336,6 +316,7 @@ gst_colorspace_srcconnect_func (GstPad *pad, GstCaps *caps, gboolean newcaps)
   GstColorspace *space;
   GstCaps *peercaps;
   GstCaps *ourcaps, *to_intersect, *try_peercaps;
+  GstPadLinkReturn res = GST_PAD_LINK_REFUSED;
   
   space = GST_COLORSPACE (gst_pad_get_parent (pad));
 
@@ -352,22 +333,40 @@ gst_colorspace_srcconnect_func (GstPad *pad, GstCaps *caps, gboolean newcaps)
    * with our incomming caps */
   peercaps = gst_caps_intersect (caps, ourcaps); 
   if (peercaps) {
-    /* see if the peer likes it too, it should as the caps say so.. */
-    if (gst_pad_try_set_caps (space->srcpad, peercaps) > 0) {
-      space->type = GST_COLORSPACE_NONE;
-      space->disabled = FALSE;
-      gst_caps_unref (peercaps);
-      return GST_PAD_LINK_DONE;
+    GstCaps *trycaps;
+
+    trycaps = peercaps;
+    while(trycaps){
+      GstCaps *caps1 = gst_caps_copy_1(trycaps);
+
+      /* see if the peer likes it too, it should as the caps say so.. */
+      if (gst_pad_try_set_caps (space->srcpad, caps1) > 0) {
+	space->type = GST_COLORSPACE_NONE;
+	space->disabled = FALSE;
+	gst_caps_unref (peercaps);
+	peercaps = caps1;
+	res = GST_PAD_LINK_DONE;
+	goto success;
+      }
+      trycaps = trycaps->next;
     }
     gst_caps_unref (peercaps);
   }
 
-  to_intersect = GST_CAPS_NEW (
+  to_intersect = gst_caps_append (
+                 GST_CAPS_NEW (
 		   "colorspace_filter",
-		   "video/raw",
-		     "width",   GST_PROPS_INT (space->width),
-		     "height",  GST_PROPS_INT (space->height)
-		 );
+		   "video/x-raw-yuv",
+		     "width",     GST_PROPS_INT (space->width),
+		     "height",    GST_PROPS_INT (space->height),
+		     "framerate", GST_PROPS_FLOAT (space->fps)
+		 ), GST_CAPS_NEW (
+		   "colorspace_filter",
+		   "video/x-raw-rgb",
+		     "width",     GST_PROPS_INT (space->width),
+		     "height",    GST_PROPS_INT (space->height),
+		     "framerate", GST_PROPS_FLOAT (space->fps)
+                 ));
 
   /* then see what the peer has that matches the size */
   peercaps = gst_caps_intersect (caps, to_intersect);
@@ -381,21 +380,33 @@ gst_colorspace_srcconnect_func (GstPad *pad, GstCaps *caps, gboolean newcaps)
   /* loop over all possibilities and select the first one we can convert and
    * is accepted by the peer */
   while (peercaps) {
-    if (colorspace_setup_converter (space, ourcaps, peercaps)) {
-      if (gst_pad_try_set_caps (space->srcpad, peercaps) > 0) {
+    GstCaps *peer1 = gst_caps_copy_1(peercaps);
+
+    if (colorspace_setup_converter (space, ourcaps, peer1)) {
+      if (gst_pad_try_set_caps (space->srcpad, peer1) > 0) {
         space->disabled = FALSE;
         gst_caps_unref (try_peercaps);
-        return GST_PAD_LINK_DONE;
+	res = GST_PAD_LINK_DONE;
+	goto success;
       }
     }
     peercaps = peercaps->next;
   }
   gst_caps_unref (try_peercaps);
+
+  if (gst_pad_recover_caps_error (space->srcpad, NULL)) {
+    res = GST_PAD_LINK_DONE;
+    goto success;
+  }
   
   /* we disable ourself here */
   space->disabled = TRUE;
+  goto done;
 
-  return GST_PAD_LINK_REFUSED;
+success:
+done:
+
+  return res;
 }
 
 GType
@@ -440,16 +451,14 @@ gst_colorspace_class_init (GstColorspaceClass *klass)
 static void
 gst_colorspace_init (GstColorspace *space)
 {
-  space->sinkpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (colorspace_sink_template_factory), "sink");
+  space->sinkpad = gst_pad_new_from_template (sinktempl, "sink");
   gst_pad_set_link_function (space->sinkpad, gst_colorspace_sinkconnect);
   gst_pad_set_getcaps_function (space->sinkpad, gst_colorspace_getcaps);
   gst_pad_set_bufferpool_function (space->sinkpad, colorspace_get_bufferpool);
   gst_pad_set_chain_function(space->sinkpad,gst_colorspace_chain);
   gst_element_add_pad(GST_ELEMENT(space),space->sinkpad);
 
-  space->srcpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (colorspace_src_template_factory), "src");
+  space->srcpad = gst_pad_new_from_template (srctempl, "src");
   gst_element_add_pad(GST_ELEMENT(space),space->srcpad);
   gst_pad_set_link_function (space->srcpad, gst_colorspace_srcconnect);
 
@@ -462,8 +471,9 @@ gst_colorspace_init (GstColorspace *space)
 }
 
 static void
-gst_colorspace_chain (GstPad *pad,GstBuffer *buf)
+gst_colorspace_chain (GstPad *pad,GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstColorspace *space;
   gint size;
   GstBuffer *outbuf = NULL;
@@ -492,6 +502,9 @@ gst_colorspace_chain (GstPad *pad,GstBuffer *buf)
     dest_bytes = ((space->destbpp+7)/8);
     src_bytes = ((space->srcbpp+7)/8);
 
+    if (!space->pool)
+      space->pool = gst_pad_get_bufferpool (space->srcpad);
+
     if (space->pool) {
       outbuf = gst_buffer_new_from_pool (space->pool, 0, 0);
     }
@@ -542,7 +555,7 @@ gst_colorspace_chain (GstPad *pad,GstBuffer *buf)
 
     gst_buffer_unref (buf);
   }
-  gst_pad_push (space->srcpad, outbuf);
+  gst_pad_push (space->srcpad, GST_DATA (outbuf));
 }
 
 static GstElementStateReturn
@@ -554,7 +567,6 @@ gst_colorspace_change_state (GstElement *element)
 
   switch (GST_STATE_TRANSITION (element)) {
     case GST_STATE_PAUSED_TO_PLAYING:
-      space->pool = gst_pad_get_bufferpool (space->srcpad);
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
       if (space->pool)
@@ -570,9 +582,7 @@ gst_colorspace_change_state (GstElement *element)
       break;
   }
 
-  parent_class->change_state (element);
-
-  return GST_STATE_SUCCESS;
+  return parent_class->change_state (element);
 }
 
 static void
@@ -610,6 +620,7 @@ static gboolean
 plugin_init (GModule *module, GstPlugin *plugin)
 {
   GstElementFactory *factory;
+  GstCaps *caps;
 #ifdef HAVE_HERMES
   gint hermes_res;
 
@@ -621,11 +632,41 @@ plugin_init (GModule *module, GstPlugin *plugin)
                                     &colorspace_details);
   g_return_val_if_fail (factory != NULL, FALSE);
   gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_PRIMARY);
-  
-  gst_element_factory_add_pad_template (factory, 
-		  GST_PAD_TEMPLATE_GET (colorspace_src_template_factory));
-  gst_element_factory_add_pad_template (factory, 
-		  GST_PAD_TEMPLATE_GET (colorspace_sink_template_factory));
+
+  /* create caps for templates */
+  caps = gst_caps_new ("csp_templ_yuv",
+                       "video/x-raw-yuv",
+	               GST_VIDEO_YUV_PAD_TEMPLATE_PROPS (
+                         GST_PROPS_LIST (
+                           GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
+	                   GST_PROPS_FOURCC (GST_STR_FOURCC ("YV12")),
+	                   GST_PROPS_FOURCC (GST_STR_FOURCC ("YUY2")))));
+  caps = gst_caps_append (caps,
+         gst_caps_new ("csp_templ_rgb24_32",
+                       "video/x-raw-rgb",
+                       GST_VIDEO_RGB_PAD_TEMPLATE_PROPS_24_32));
+  caps = gst_caps_append (caps,
+         gst_caps_new ("csp_templ_rgb15",
+                       "video/x-raw-rgb",
+                       GST_VIDEO_RGB_PAD_TEMPLATE_PROPS_15));
+  caps = gst_caps_append (caps,
+         gst_caps_new ("csp_templ_rgb16",
+                       "video/x-raw-rgb",
+                       GST_VIDEO_RGB_PAD_TEMPLATE_PROPS_16));
+
+  /* build templates */
+  srctempl  = gst_pad_template_new ("src",
+				    GST_PAD_SRC,
+				    GST_PAD_ALWAYS,
+				    caps, NULL);
+  gst_caps_ref (caps);
+  sinktempl = gst_pad_template_new ("sink",
+				    GST_PAD_SINK,
+				    GST_PAD_ALWAYS,
+				    caps, NULL);
+
+  gst_element_factory_add_pad_template (factory, srctempl);
+  gst_element_factory_add_pad_template (factory, sinktempl);
 
   gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
 
