@@ -17,6 +17,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include "string.h"
 #include "gstlame.h"
 
 /* elementfactory information */
@@ -24,7 +28,7 @@ static GstElementDetails gst_lame_details =
 {
   "L.A.M.E. mp3 encoder",
   "Codec/Audio/Encoder",
-  "GPL",
+  "LGPL",
   "High-quality free MP3 encoder",
   VERSION,
   "Erik Walthinsen <omega@cse.ogi.edu>",
@@ -37,9 +41,7 @@ GST_PAD_TEMPLATE_FACTORY (gst_lame_sink_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "gstlame_sink",
-    "audio/raw",
-      "format",     GST_PROPS_STRING ("int"),
-      "law",        GST_PROPS_INT (0),
+    "audio/x-raw-int",
       "endianness", GST_PROPS_INT (G_BYTE_ORDER),
       "signed",     GST_PROPS_BOOLEAN (TRUE),
       "width",      GST_PROPS_INT (16),
@@ -65,8 +67,21 @@ GST_PAD_TEMPLATE_FACTORY (gst_lame_src_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "gstlame_src",
-    "audio/x-mp3",
-    NULL
+    "audio/mpeg",
+      "mpegversion", GST_PROPS_INT (1),
+      "layer",      GST_PROPS_INT (3),
+      "rate",       GST_PROPS_LIST (
+	              GST_PROPS_INT (8000), 
+	              GST_PROPS_INT (11025), 
+	              GST_PROPS_INT (12000), 
+	              GST_PROPS_INT (16000), 
+	              GST_PROPS_INT (22050),
+	              GST_PROPS_INT (24000),
+	              GST_PROPS_INT (32000),
+	              GST_PROPS_INT (44100),
+	              GST_PROPS_INT (48000)
+	            ),
+      "channels",   GST_PROPS_INT_RANGE (1, 2)
   )
 )
 
@@ -170,6 +185,7 @@ enum {
   ARG_ALLOW_DIFF_SHORT,
   ARG_NO_SHORT_BLOCKS,
   ARG_EMPHASIS,
+  ARG_METADATA,
 };
 
 
@@ -180,7 +196,7 @@ static void			gst_lame_set_property	(GObject *object, guint prop_id,
 		 					 const GValue *value, GParamSpec *pspec);
 static void			gst_lame_get_property	(GObject *object, guint prop_id, 
 							 GValue *value, GParamSpec *pspec);
-static void			gst_lame_chain		(GstPad *pad, GstBuffer *buf);
+static void			gst_lame_chain		(GstPad *pad, GstData *_data);
 static gboolean 		gst_lame_setup 		(GstLame *lame);
 static GstElementStateReturn 	gst_lame_change_state 	(GstElement *element);
 
@@ -320,23 +336,29 @@ gst_lame_class_init (GstLameClass *klass)
     g_param_spec_boolean ("emphasis", "Emphasis", "Emphasis",
                           TRUE, G_PARAM_READWRITE));
 
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_METADATA,
+    g_param_spec_boxed ("metadata", "Metadata", "Metadata to add to the stream,",
+            GST_TYPE_CAPS, G_PARAM_READWRITE));
+
+
   gobject_class->set_property = gst_lame_set_property;
   gobject_class->get_property = gst_lame_get_property;
 
   gstelement_class->change_state = gst_lame_change_state;
 }
 
-static GstPadConnectReturn
+static GstPadLinkReturn
 gst_lame_sinkconnect (GstPad *pad, GstCaps *caps)
 {
   GstLame *lame;
+  gint out_samplerate;
 
   lame = GST_LAME (gst_pad_get_parent (pad));
 
   if (!GST_CAPS_IS_FIXED (caps)) {
-    GST_DEBUG (GST_CAT_CAPS, "caps on lame pad %s:%s not fixed, delayed",
+    GST_DEBUG ("caps on lame pad %s:%s not fixed, delayed",
 	       GST_DEBUG_PAD_NAME (pad));
-    return GST_PAD_CONNECT_DELAYED;
+    return GST_PAD_LINK_DELAYED;
   }
 
   gst_caps_get_int (caps, "rate", &lame->samplerate);
@@ -345,28 +367,36 @@ gst_lame_sinkconnect (GstPad *pad, GstCaps *caps)
   if (!gst_lame_setup (lame)) {
     gst_element_error (GST_ELEMENT (lame), 
 	               "could not initialize encoder (wrong parameters?)");
-    return GST_PAD_CONNECT_REFUSED;
+    return GST_PAD_LINK_REFUSED;
   }
 
-  return GST_PAD_CONNECT_OK;
+  out_samplerate = lame_get_out_samplerate (lame->lgf);
+  caps = GST_CAPS_NEW ("lame_src_caps",
+		       "audio/mpeg",
+                         "mpegversion", GST_PROPS_INT (1),
+			 "layer",    GST_PROPS_INT (3),
+			 "channels", GST_PROPS_INT (lame->num_channels),
+			 "rate",     GST_PROPS_INT (out_samplerate));
+
+  return gst_pad_try_set_caps (lame->srcpad, caps);
 }
 
 static void
 gst_lame_init (GstLame *lame)
 {
-  GST_DEBUG_ENTER ("(\"%s\")", gst_element_get_name (GST_ELEMENT (lame)));
+  GST_DEBUG_OBJECT (lame, "starting initialization");
 
   lame->sinkpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (gst_lame_sink_factory), "sink");
   gst_element_add_pad (GST_ELEMENT (lame), lame->sinkpad);
   gst_pad_set_chain_function (lame->sinkpad, gst_lame_chain);
-  gst_pad_set_connect_function (lame->sinkpad, gst_lame_sinkconnect);
+  gst_pad_set_link_function (lame->sinkpad, gst_lame_sinkconnect);
 
   lame->srcpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (gst_lame_src_factory), "src");
   gst_element_add_pad (GST_ELEMENT (lame), lame->srcpad);
 
   GST_FLAG_SET (lame, GST_ELEMENT_EVENT_AWARE);
 
-  GST_DEBUG (GST_CAT_PLUGIN_INFO, "setting up lame encoder");
+  GST_DEBUG ("setting up lame encoder");
   lame->lgf = lame_init ();
 
   lame->samplerate = 44100;
@@ -405,9 +435,64 @@ gst_lame_init (GstLame *lame)
   lame->no_short_blocks = lame_get_no_short_blocks (lame->lgf);
   lame->emphasis = lame_get_emphasis (lame->lgf);
 
-  GST_DEBUG (GST_CAT_PLUGIN_INFO, "done initializing lame element");
+  lame->metadata = GST_CAPS_NEW (
+      "lame_metadata",
+      "application/x-gst-metadata",
+      "comment",     GST_PROPS_STRING ("Track encoded with GStreamer"),
+      "year",        GST_PROPS_STRING (""),
+      "tracknumber", GST_PROPS_STRING (""),
+      "title",       GST_PROPS_STRING (""),
+      "artist",      GST_PROPS_STRING (""),
+      "album",       GST_PROPS_STRING (""),
+      "genre",       GST_PROPS_STRING ("")
+  );
+
+  id3tag_init (lame->lgf);
+  
+  GST_DEBUG_OBJECT (lame, "done initializing");
 }
 
+static void
+gst_lame_add_metadata (GstLame *lame, GstCaps *caps)
+{
+  GList *props;
+  GstPropsEntry *prop;
+
+  if (caps == NULL)
+    return;
+
+  props = gst_caps_get_props (caps)->properties;
+  while (props) {
+    prop = (GstPropsEntry*)(props->data);
+    props = g_list_next(props);
+
+    if (gst_props_entry_get_props_type (prop) == GST_PROPS_STRING_TYPE) {
+      const gchar *name = gst_props_entry_get_name (prop);
+      const gchar *value;
+
+      gst_props_entry_get_string (prop, &value);
+
+      if (!value || strlen (value) == 0)
+        continue;
+
+      if (strcmp (name, "comment") == 0) {
+        id3tag_set_comment (lame->lgf, value);
+      } else if (strcmp (name, "date") == 0) {
+        id3tag_set_year (lame->lgf, value);
+      } else if (strcmp (name, "tracknumber") == 0) {
+        id3tag_set_track (lame->lgf, value);
+      } else if (strcmp (name, "title") == 0) {
+        id3tag_set_title (lame->lgf, value);
+      } else if (strcmp (name, "artist") == 0) {
+        id3tag_set_artist (lame->lgf, value);
+      } else if (strcmp (name, "album") == 0) {
+        id3tag_set_album (lame->lgf, value);
+      } else if (strcmp (name, "genre") == 0) {
+        id3tag_set_genre (lame->lgf, value);
+      }
+    }
+  }
+}
 
 static void
 gst_lame_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
@@ -510,6 +595,9 @@ gst_lame_set_property (GObject *object, guint prop_id, const GValue *value, GPar
     case ARG_EMPHASIS:
       lame->emphasis = g_value_get_boolean (value);
       break;
+    case ARG_METADATA:
+      lame->metadata = g_value_get_boxed (value);
+      break;
     default:
       break;
   }
@@ -617,6 +705,9 @@ gst_lame_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec
     case ARG_EMPHASIS:
       g_value_set_boolean (value, lame->emphasis);
       break;
+    case ARG_METADATA:
+      g_value_set_static_boxed (value, lame->metadata);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -624,8 +715,9 @@ gst_lame_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec
 }
 
 static void
-gst_lame_chain (GstPad *pad, GstBuffer *buf)
+gst_lame_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstLame *lame;
   GstBuffer *outbuf;
   gchar *mp3_data = NULL;
@@ -634,7 +726,7 @@ gst_lame_chain (GstPad *pad, GstBuffer *buf)
 
   lame = GST_LAME (gst_pad_get_parent (pad));
 
-  GST_DEBUG (GST_CAT_PLUGIN_INFO, "entered chain");
+  GST_DEBUG ("entered chain");
 
   if (GST_IS_EVENT (buf)) {
     switch (GST_EVENT_TYPE (buf)) {
@@ -643,8 +735,8 @@ gst_lame_chain (GstPad *pad, GstBuffer *buf)
       case GST_EVENT_FLUSH:
         mp3_buffer_size = 7200;
         mp3_data = g_malloc (mp3_buffer_size);
-	
-        mp3_size = lame_encode_flush_nogap (lame->lgf, mp3_data, mp3_buffer_size);
+
+        mp3_size = lame_encode_flush (lame->lgf, mp3_data, mp3_buffer_size);
 	gst_event_unref (GST_EVENT (buf));
         break;	
       default:
@@ -653,6 +745,8 @@ gst_lame_chain (GstPad *pad, GstBuffer *buf)
     }
   }
   else {
+    gint64 duration;
+
     if (!lame->initialized) {
       gst_buffer_unref (buf);
       gst_element_error (GST_ELEMENT (lame), "encoder not initialized (input is not audio?)");
@@ -677,26 +771,49 @@ gst_lame_chain (GstPad *pad, GstBuffer *buf)
 		    mp3_data, mp3_buffer_size);
     }
 
-    GST_DEBUG (GST_CAT_PLUGIN_INFO, 
+    GST_DEBUG (
 	       "encoded %d bytes of audio to %d bytes of mp3", 
 	       GST_BUFFER_SIZE (buf), mp3_size);
 
+    duration = (GST_SECOND * GST_BUFFER_SIZE (buf) /
+		(2 * lame->samplerate * lame->num_channels));
+
+    if (GST_BUFFER_DURATION (buf) != GST_CLOCK_TIME_NONE &&
+	GST_BUFFER_DURATION (buf) != duration)
+      GST_DEBUG (
+		 "mad: incoming buffer had incorrect duration %lld, "
+		 "outgoing buffer will have correct duration %lld",
+		 GST_BUFFER_DURATION (buf), duration);
+
+    if (lame->last_ts == GST_CLOCK_TIME_NONE) {
+      lame->last_ts       = GST_BUFFER_TIMESTAMP (buf);
+      lame->last_offs     = GST_BUFFER_OFFSET (buf);
+      lame->last_duration = duration;
+    } else {
+      lame->last_duration += duration;
+    }
+
     gst_buffer_unref (buf);
   }
   
   if (mp3_size > 0) {
     outbuf = gst_buffer_new ();
-    GST_BUFFER_DATA (outbuf) = mp3_data;
-    GST_BUFFER_SIZE (outbuf) = mp3_size;
+    GST_BUFFER_DATA (outbuf)      = mp3_data;
+    GST_BUFFER_SIZE (outbuf)      = mp3_size;
+    GST_BUFFER_TIMESTAMP (outbuf) = lame->last_ts;
+    GST_BUFFER_OFFSET (outbuf)    = lame->last_offs;
+    GST_BUFFER_DURATION (outbuf)  = lame->last_duration;
 
-    gst_pad_push (lame->srcpad,outbuf);
+    gst_pad_push (lame->srcpad,GST_DATA (outbuf));
+
+    lame->last_ts = GST_CLOCK_TIME_NONE;
   }
   else { 
     g_free (mp3_data);
   }
 
   if (eos) {
-    gst_pad_push (lame->srcpad, GST_BUFFER (gst_event_new (GST_EVENT_EOS)));
+    gst_pad_push (lame->srcpad, GST_DATA (gst_event_new (GST_EVENT_EOS)));
     gst_element_set_eos (GST_ELEMENT (lame));
   }
 }
@@ -705,7 +822,7 @@ gst_lame_chain (GstPad *pad, GstBuffer *buf)
 static gboolean
 gst_lame_setup (GstLame *lame)
 {
-  GST_DEBUG_ENTER ("(\"%s\")", gst_element_get_name (GST_ELEMENT (lame)));
+  GST_DEBUG_OBJECT (lame, "starting setup");
 
   /* check if we're already initialized; if we are, we might want to check
    * if this initialization is compatible with the previous one */
@@ -751,6 +868,8 @@ gst_lame_setup (GstLame *lame)
   lame_set_no_short_blocks (lame->lgf, lame->no_short_blocks);
   lame_set_emphasis (lame->lgf, lame->emphasis);
 
+  gst_lame_add_metadata (lame, lame->metadata);
+
   /* initialize the lame encoder */
   if (lame_init_params (lame->lgf) < 0) {
     lame->initialized = FALSE;
@@ -758,12 +877,12 @@ gst_lame_setup (GstLame *lame)
   else {
     lame->initialized = TRUE;
     /* FIXME: it would be nice to print out the mode here */
-    GST_INFO (GST_CAT_PLUGIN_INFO, 
+    GST_INFO ( 
 	      "lame encoder initialized (%d kbit/s, %d Hz, %d channels)", 
 	      lame->bitrate, lame->samplerate, lame->num_channels);
   }
 
-  GST_DEBUG_LEAVE ("");
+  GST_DEBUG_OBJECT (lame, "done with setup");
 
   return lame->initialized;
 }
@@ -777,9 +896,12 @@ gst_lame_change_state (GstElement *element)
 
   lame = GST_LAME (element);
 
-  GST_DEBUG (0,"state pending %d", GST_STATE_PENDING (element));
+  GST_DEBUG ("state pending %d", GST_STATE_PENDING (element));
 
   switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_READY_TO_PAUSED:
+      lame->last_ts = GST_CLOCK_TIME_NONE;
+      break;
     case GST_STATE_READY_TO_NULL:
       if (lame->initialized) {
         lame_close (lame->lgf);
