@@ -16,6 +16,9 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 #include <gst/gst.h>
 #include "gstpngenc.h"
@@ -24,10 +27,10 @@
 
 
 GstElementDetails gst_pngenc_details = {
-  "",
-  "Codec/Image/Encoder",
+  "PNG encoder",
+  "Codec/Image",
   "LGPL",
-  "Encode a video frame to a .png image"
+  "Encode a video frame to a .png image",
   VERSION,
   "Jeremy SIMON <jsimon13@yahoo.fr>",
   "(C) 2000 Donald Graft",
@@ -49,7 +52,7 @@ enum
 static void	gst_pngenc_class_init	(GstPngEncClass *klass);
 static void	gst_pngenc_init		(GstPngEnc *pngenc);
 
-static void	gst_pngenc_chain	(GstPad *pad, GstBuffer *buf);
+static void	gst_pngenc_chain	(GstPad *pad, GstData *_data);
 
 static GstElementClass *parent_class = NULL;
 
@@ -104,6 +107,7 @@ static GstPadLinkReturn
 gst_pngenc_sinklink (GstPad *pad, GstCaps *caps)
 {
   GstPngEnc *pngenc;
+  gfloat fps;
 
   pngenc = GST_PNGENC (gst_pad_get_parent (pad));
 
@@ -112,8 +116,15 @@ gst_pngenc_sinklink (GstPad *pad, GstCaps *caps)
 
   gst_caps_get_int (caps, "width", &pngenc->width);
   gst_caps_get_int (caps, "height", &pngenc->height);
+  gst_caps_get_float (caps, "framerate", &fps);
   gst_caps_get_int (caps, "bpp", &pngenc->bpp);
 
+  caps = GST_CAPS_NEW ("png_src",
+		       "video/x-png",
+			 "framerate", GST_PROPS_FLOAT (fps),
+			 "width",     GST_PROPS_INT (pngenc->width),
+			 "height",    GST_PROPS_INT (pngenc->height));
+
   return gst_pad_try_set_caps (pngenc->srcpad, caps);
 }
 
@@ -140,7 +151,7 @@ GstPngEnc *pngenc;
 
   pngenc = (GstPngEnc *) png_get_io_ptr (png_ptr);
 
-  gst_pad_push (pngenc->srcpad, GST_BUFFER (gst_event_new (GST_EVENT_FLUSH)));
+  gst_pad_push (pngenc->srcpad, GST_DATA (gst_event_new (GST_EVENT_FLUSH)));
 }
 
 
@@ -165,8 +176,9 @@ void user_write_data (png_structp png_ptr, png_bytep data, png_uint_32 length)
 }
 
 static void
-gst_pngenc_chain (GstPad *pad, GstBuffer *buf)
+gst_pngenc_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstPngEnc *pngenc;
   gint row_index;
   png_byte *row_pointers[MAX_HEIGHT];
@@ -198,7 +210,7 @@ gst_pngenc_chain (GstPad *pad, GstBuffer *buf)
   /* non-0 return is from a longjmp inside of libpng */
   if (setjmp (pngenc->png_struct_ptr->jmpbuf) != 0)
   {
-    GST_DEBUG (GST_CAT_PLUGIN_INFO, "returning from longjmp");
+    GST_DEBUG ("returning from longjmp");
     png_destroy_write_struct (&pngenc->png_struct_ptr, &pngenc->png_info_ptr);
     return;
   }
@@ -235,10 +247,11 @@ gst_pngenc_chain (GstPad *pad, GstBuffer *buf)
   png_destroy_info_struct (pngenc->png_struct_ptr, &pngenc->png_info_ptr);
   png_destroy_write_struct (&pngenc->png_struct_ptr, (png_infopp) NULL);
 
-  gst_pad_push (pngenc->srcpad, pngenc->buffer_out);
-  /* send NEW MEDIA event, since a frame has been pushed out */
-  event = gst_event_new (GST_EVENT_NEW_MEDIA);
-  gst_pad_push (pngenc->srcpad, GST_BUFFER (event));
+  gst_pad_push (pngenc->srcpad, GST_DATA (pngenc->buffer_out));
+
+  /* send EOS event, since a frame has been pushed out */
+  event = gst_event_new (GST_EVENT_EOS);
+  gst_pad_push (pngenc->srcpad, GST_DATA (event));
 
   gst_buffer_unref (buf);
 }
