@@ -17,12 +17,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "gstshout.h"
 
 /* elementfactory information */
 static GstElementDetails icecastsend_details = {
   "An Icecast plugin",
-  "Sink/Network/Icecast",
+  "Sink/Network",
   "GPL",
   "Sends data to an icecast server using libshout",
   VERSION,
@@ -75,8 +78,11 @@ sink_template_factory (void)
       GST_PAD_ALWAYS,
       gst_caps_new (
         "icecastsend_sink",
-        "audio/x-mp3",
-	NULL),
+        "audio/mpeg",
+	gst_props_new (
+	  "layer", GST_PROPS_INT_RANGE (1, 3),
+	  NULL
+	)),
       NULL);
   }
 
@@ -86,7 +92,7 @@ sink_template_factory (void)
 static void		gst_icecastsend_class_init	(GstIcecastSendClass *klass);
 static void		gst_icecastsend_init		(GstIcecastSend *icecastsend);
 
-static void		gst_icecastsend_chain		(GstPad *pad, GstBuffer *buf);
+static void		gst_icecastsend_chain		(GstPad *pad, GstData *_data);
 
 static void		gst_icecastsend_set_property	(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void		gst_icecastsend_get_property	(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
@@ -209,8 +215,9 @@ gst_icecastsend_init (GstIcecastSend *icecastsend)
 }
 
 static void
-gst_icecastsend_chain (GstPad *pad, GstBuffer *buf)
+gst_icecastsend_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstIcecastSend *icecastsend;
   glong ret;
 
@@ -393,11 +400,11 @@ gst_icecastsend_change_state (GstElement *element)
 
   icecastsend = GST_ICECASTSEND(element);
 
-  GST_DEBUG (0,"state pending %d", GST_STATE_PENDING (element));
+  GST_DEBUG ("state pending %d", GST_STATE_PENDING (element));
 
   /* if going down into NULL state, close the file if it's open */
   switch (GST_STATE_TRANSITION (element)) {
-   case GST_STATE_NULL_TO_READY:
+   case GST_STATE_READY_TO_NULL:
      shout_init_connection (&icecastsend->conn);
 
      /* --- FIXME: shout requires an ip, and fails if it is given a host. */
@@ -422,13 +429,14 @@ gst_icecastsend_change_state (GstElement *element)
        g_print ("connected to server...\n");
      }
      else {
-       /* changed from g_warning, and included result code lookup. */
-       g_warning ("couldn't connect to server... (%i: %s)\n", icecastsend->conn.error, SHOUT_ERRORS[icecastsend->conn.error]);
+       g_warning ("couldn't connect to server... (%i: %s)\n", 
+		  icecastsend->conn.error, 
+		  SHOUT_ERRORS[icecastsend->conn.error]);
        shout_disconnect (&icecastsend->conn);
        return GST_STATE_FAILURE;
      }
      break;
-   case GST_STATE_READY_TO_NULL:
+   case GST_STATE_PAUSED_TO_READY:
      shout_disconnect (&icecastsend->conn);
      break;
    default:
