@@ -24,7 +24,7 @@
 
 #include <config.h>
 #include <gst/gst.h>
-#include <gst/bytestream/bytestream.h>
+#include <gst/gstbytestream.h>
 #include <gst/riff/riff.h>
 
 #ifdef __cplusplus
@@ -110,12 +110,13 @@ struct _GstAviDemux {
   guint 	 num_streams;
   guint 	 num_v_streams;
   guint 	 num_a_streams;
-
   avi_stream_context stream[GST_AVI_DEMUX_MAX_STREAMS];
 
   gboolean 	 seek_pending;
   gint64 	 seek_offset;
   guint64 	 last_seek;
+
+  GstCaps	*metadata, *streaminfo;
 };
 
 struct _GstAviDemuxClass {
