@@ -25,6 +25,9 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <config.h>
 
 #include <stdlib.h>
@@ -81,43 +84,74 @@ GST_PAD_TEMPLATE_FACTORY (video_sink_factory,
   GST_PAD_SINK,
   GST_PAD_REQUEST,
   GST_CAPS_NEW (
-    "avimux_sink_video",
-    "video/avi",
-      "format",   GST_PROPS_STRING ("strf_vids")
-  ),
-  GST_CAPS_NEW (
-    "avimux_sink_video",
-    "video/raw",
+    "avimux_sink_video_yuv",
+    "video/x-raw-yuv",
       "format", GST_PROPS_LIST (
                   GST_PROPS_FOURCC (GST_MAKE_FOURCC('Y','U','Y','2')),
-                  GST_PROPS_FOURCC (GST_MAKE_FOURCC('I','4','2','0')),
-                  GST_PROPS_FOURCC (GST_MAKE_FOURCC('Y','4','1','P'))
+                  GST_PROPS_FOURCC (GST_MAKE_FOURCC('I','4','2','0'))
                 ),
       "width",  GST_PROPS_INT_RANGE (16, 4096),
       "height", GST_PROPS_INT_RANGE (16, 4096)
   ),
   GST_CAPS_NEW (
-    "avimux_sink_video",
-    "video/raw",
-      "format", GST_PROPS_FOURCC (GST_MAKE_FOURCC('R','G','B',' ')),
+    "avimux_sink_video_jpeg",
+    "video/x-jpeg",
       "width",  GST_PROPS_INT_RANGE (16, 4096),
-      "height", GST_PROPS_INT_RANGE (16, 4096),
-      "depth",  GST_PROPS_LIST(
-                  GST_PROPS_INT(16),
-                  GST_PROPS_INT(16),
-                  GST_PROPS_INT(24),
-                  GST_PROPS_INT(32)
-                ),
-      "bpp",    GST_PROPS_LIST(
-                  GST_PROPS_INT(15),
-                  GST_PROPS_INT(16),
-                  GST_PROPS_INT(24),
-                  GST_PROPS_INT(32)
-                )
+      "height", GST_PROPS_INT_RANGE (16, 4096)
   ),
   GST_CAPS_NEW (
-    "avimux_sink_video",
-    "video/jpeg",
+    "avimux_sink_video_divx",
+    "video/x-divx",
+      "width",  GST_PROPS_INT_RANGE (16, 4096),
+      "height", GST_PROPS_INT_RANGE (16, 4096),
+      "divxversion", GST_PROPS_INT_RANGE (3, 5)
+  ),
+  GST_CAPS_NEW (
+    "avimux_sink_video_xvid",
+    "video/x-xvid",
+      "width",  GST_PROPS_INT_RANGE (16, 4096),
+      "height", GST_PROPS_INT_RANGE (16, 4096)
+  ),
+  GST_CAPS_NEW (
+    "avimux_sink_video_3ivx",
+    "video/x-3ivx",
+      "width",  GST_PROPS_INT_RANGE (16, 4096),
+      "height", GST_PROPS_INT_RANGE (16, 4096)
+  ),
+  GST_CAPS_NEW (
+    "avimux_sink_video_msmpeg",
+    "video/x-msmpeg",
+      "width",  GST_PROPS_INT_RANGE (16, 4096),
+      "height", GST_PROPS_INT_RANGE (16, 4096),
+      "msmpegversion", GST_PROPS_INT_RANGE (41, 43)
+  ),
+  GST_CAPS_NEW (
+    "avimux_sink_video_mpeg",
+    "video/mpeg",
+      "width",  GST_PROPS_INT_RANGE (16, 4096),
+      "height", GST_PROPS_INT_RANGE (16, 4096),
+      "mpegversion", GST_PROPS_INT (1),
+      "systemstream", GST_PROPS_BOOLEAN (FALSE)
+  ),
+  GST_CAPS_NEW (
+    "avimux_sink_video_h263",
+    "video/x-h263",
+      "width",  GST_PROPS_INT_RANGE (16, 4096),
+      "height", GST_PROPS_INT_RANGE (16, 4096)
+  ),
+  GST_CAPS_NEW (
+    "avimux_sink_video_dv",
+    "video/x-dv",
+      "width",  GST_PROPS_INT (720),
+      "height", GST_PROPS_LIST (
+		  GST_PROPS_INT (576),
+		  GST_PROPS_INT (480)
+		),
+      "systemstream", GST_PROPS_BOOLEAN (FALSE)
+  ),
+  GST_CAPS_NEW (
+    "avimux_sink_video_hfyu",
+    "video/x-huffyuv",
       "width",  GST_PROPS_INT_RANGE (16, 4096),
       "height", GST_PROPS_INT_RANGE (16, 4096)
   )
@@ -128,16 +162,9 @@ GST_PAD_TEMPLATE_FACTORY (audio_sink_factory,
   GST_PAD_SINK,
   GST_PAD_REQUEST,
   GST_CAPS_NEW (
-    "avimux_sink_audio",
-    "video/avi",
-      "format",   GST_PROPS_STRING ("strf_auds")
-  ),
-  GST_CAPS_NEW (
-    "avimux_sink_audio",
-    "audio/raw",
-      "format",           GST_PROPS_STRING ("int"),
-      "law",              GST_PROPS_INT (0),
-      "endianness",       GST_PROPS_INT (G_BYTE_ORDER),
+    "avimux_sink_audio_raw",
+    "audio/x-raw-int",
+      "endianness",       GST_PROPS_INT (G_LITTLE_ENDIAN),
       "signed",           GST_PROPS_LIST (
       			    GST_PROPS_BOOLEAN (TRUE),
       			    GST_PROPS_BOOLEAN (FALSE)
@@ -150,18 +177,27 @@ GST_PAD_TEMPLATE_FACTORY (audio_sink_factory,
       			    GST_PROPS_INT (8),
       			    GST_PROPS_INT (16)
 			  ),
-      "rate",             GST_PROPS_INT_RANGE (1000, 48000),
+      "rate",             GST_PROPS_INT_RANGE (1000, 96000),
       "channels",         GST_PROPS_INT_RANGE (1, 2)
   ),
   GST_CAPS_NEW (
-    "avimux_sink_audio",
-    "audio/x-mp3",
-      NULL
+    "avimux_sink_audio_mpeg",
+    "audio/mpeg",
+      "layer",		  GST_PROPS_INT_RANGE (1, 3),
+      "rate",             GST_PROPS_INT_RANGE (1000, 96000),
+      "channels",         GST_PROPS_INT_RANGE (1, 2)
   ),
   GST_CAPS_NEW (
-    "avimux_sink_audio",
-    "application/x-ogg",
-      NULL
+    "avimux_sink_audio_vorbis",
+    "audio/x-vorbis",
+      "rate",             GST_PROPS_INT_RANGE (1000, 96000),
+      "channels",         GST_PROPS_INT_RANGE (1, 2)
+  ),
+  GST_CAPS_NEW (
+    "avimux_sink_audio_ac3",
+    "audio/x-ac3",
+      "rate",             GST_PROPS_INT_RANGE (1000, 96000),
+      "channels",         GST_PROPS_INT_RANGE (1, 6)
   )
 )
     
@@ -222,7 +258,8 @@ gst_avimux_class_init (GstAviMuxClass *klass)
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_BIGFILE,
-    g_param_spec_boolean("bigfile","Bigfile Support","Whether to capture large or small AVI files",
+    g_param_spec_boolean("bigfile","Bigfile Support",
+			 "Support for openDML-2.0 (big) AVI files",
     0,G_PARAM_READWRITE));
 
   gstelement_class->request_new_pad = gst_avimux_request_new_pad;
@@ -236,19 +273,12 @@ gst_avimux_class_init (GstAviMuxClass *klass)
 static const GstEventMask *
 gst_avimux_get_event_masks (GstPad *pad)
 {
-  static const GstEventMask gst_avimux_src_event_masks[] = {
-    { GST_EVENT_NEW_MEDIA, 0 },
-    { 0, }
-  };
   static const GstEventMask gst_avimux_sink_event_masks[] = {
     { GST_EVENT_EOS, 0 },
     { 0, }
   };
 
-  if (GST_PAD_IS_SRC(pad))
-    return gst_avimux_src_event_masks;
-  else
-    return gst_avimux_sink_event_masks;
+  return gst_avimux_sink_event_masks;
 }
 
 static void 
@@ -259,8 +289,6 @@ gst_avimux_init (GstAviMux *avimux)
   gst_element_add_pad (GST_ELEMENT (avimux), avimux->srcpad);
 
   GST_FLAG_SET (GST_ELEMENT(avimux), GST_ELEMENT_EVENT_AWARE);
-  gst_pad_set_event_function(avimux->srcpad, gst_avimux_handle_event);
-  gst_pad_set_event_mask_function(avimux->srcpad, gst_avimux_get_event_masks);
 
   avimux->audiosinkpad = NULL;
   avimux->audio_pad_connected = FALSE;
@@ -280,7 +308,10 @@ gst_avimux_init (GstAviMux *avimux)
   memset(&(avimux->auds),0,sizeof(gst_riff_strf_auds));
   avimux->vids_hdr.type = GST_MAKE_FOURCC('v','i','d','s');
   avimux->vids_hdr.rate = 1000000;
+  avimux->avi_hdr.max_bps = 10000000;
   avimux->auds_hdr.type = GST_MAKE_FOURCC('a','u','d','s');
+  avimux->vids_hdr.quality = 0xFFFFFFFF;
+  avimux->auds_hdr.quality = 0xFFFFFFFF;
 
   avimux->idx = NULL;
 
@@ -288,13 +319,11 @@ gst_avimux_init (GstAviMux *avimux)
 
   avimux->enable_large_avi = TRUE;
 
-  avimux->framerate = 0;
-
   gst_element_set_loop_function(GST_ELEMENT(avimux), gst_avimux_loop);
 }
 
-static GstPadConnectReturn
-gst_avimux_sinkconnect (GstPad *pad, GstCaps *vscaps)
+static GstPadLinkReturn
+gst_avimux_vidsinkconnect (GstPad *pad, GstCaps *vscaps)
 {
   GstAviMux *avimux;
   GstCaps *caps;
@@ -303,148 +332,197 @@ gst_avimux_sinkconnect (GstPad *pad, GstCaps *vscaps)
 
   /* we are not going to act on variable caps */
   if (!GST_CAPS_IS_FIXED (vscaps))
-    return GST_PAD_CONNECT_DELAYED;
+    return GST_PAD_LINK_DELAYED;
 
-  GST_DEBUG (0, "avimux: sinkconnect triggered on %s", gst_pad_get_name (pad));
+  GST_DEBUG ("avimux: video sinkconnect triggered on %s",
+	     gst_pad_get_name (pad));
+
+  for (caps = vscaps; caps != NULL; caps = vscaps = vscaps->next)
+  {
+    const gchar* mimetype = gst_caps_get_mime(caps);
+    gfloat fps = 0.;
+
+    /* global */
+    avimux->vids.size        = sizeof(gst_riff_strf_vids);
+    avimux->vids.planes      = 1;
+    gst_caps_get (caps, "width", &avimux->vids.width,
+			"height", &avimux->vids.height,
+			"framerate", &fps,
+			NULL);
+    if (fps != 0.)
+      avimux->vids_hdr.scale = avimux->vids_hdr.rate / fps;
+
+    if (!strcmp (mimetype, "video/x-raw-yuv"))
+    {
+      guint32 format;
+
+      gst_caps_get_fourcc_int (caps, "format", &format);
+      avimux->vids.compression = format;
+      switch (format)
+      {
+        case GST_MAKE_FOURCC('Y','U','Y','2'):
+          avimux->vids.bit_cnt     = 16;
+          break;
+        case GST_MAKE_FOURCC('I','4','2','0'):
+          avimux->vids.bit_cnt     = 12;
+          break;
+      }
+
+      goto done;
+    }
+    else
+    {
+      avimux->vids.bit_cnt = 24;
+      avimux->vids.compression = 0;
+
+      /* find format */
+      if (!strcmp (mimetype, "video/x-huffyuv")) {
+        avimux->vids.compression = GST_MAKE_FOURCC('H','F','Y','U');
+      } else if (!strcmp (mimetype, "video/x-jpeg")) {
+        avimux->vids.compression = GST_MAKE_FOURCC('M','J','P','G');
+      } else if (!strcmp (mimetype, "video/x-divx")) {
+        gint divxversion;
+        gst_caps_get_int (caps, "divxversion", &divxversion);
+        switch (divxversion) {
+          case 3:
+            avimux->vids.compression = GST_MAKE_FOURCC('D','I','V','3');
+            break;
+          case 4:
+            avimux->vids.compression = GST_MAKE_FOURCC('D','I','V','X');
+            break;
+          case 5:
+            avimux->vids.compression = GST_MAKE_FOURCC('D','X','5','0');
+            break;
+        }
+      } else if (!strcmp (mimetype, "video/x-xvid")) {
+        avimux->vids.compression = GST_MAKE_FOURCC('X','V','I','D');
+      } else if (!strcmp (mimetype, "video/x-3ivx")) {
+        avimux->vids.compression = GST_MAKE_FOURCC('3','I','V','2');
+      } else if (!strcmp (mimetype, "video/x-msmpeg")) {
+        gint msmpegversion;
+        gst_caps_get_int (caps, "msmpegversion", &msmpegversion);
+        switch (msmpegversion) {
+          case 41:
+            avimux->vids.compression = GST_MAKE_FOURCC('M','P','G','4');
+            break;
+          case 42:
+            avimux->vids.compression = GST_MAKE_FOURCC('M','P','4','2');
+            break;
+          case 43:
+            avimux->vids.compression = GST_MAKE_FOURCC('M','P','4','3');
+            break;
+        }
+      } else if (!strcmp (mimetype, "video/x-dv")) {
+        avimux->vids.compression = GST_MAKE_FOURCC('D','V','S','D');
+      } else if (!strcmp (mimetype, "video/x-h263")) {
+        avimux->vids.compression = GST_MAKE_FOURCC('H','2','6','3');
+      } else if (!strcmp (mimetype, "video/mpeg")) {
+        avimux->vids.compression = GST_MAKE_FOURCC('M','P','E','G');
+      }
+
+      if (!avimux->vids.compression) {
+        continue;
+      }
+
+      goto done;
+    }
+  }
+  return GST_PAD_LINK_REFUSED;
+
+done:
+  avimux->vids_hdr.fcc_handler = avimux->vids.compression;
+  avimux->vids.image_size = avimux->vids.height * avimux->vids.width;
+  avimux->avi_hdr.width = avimux->vids.width;
+  avimux->avi_hdr.height = avimux->vids.height;
+  avimux->avi_hdr.us_frame = avimux->vids_hdr.scale;
+  return GST_PAD_LINK_OK;
+}
+
+static GstPadLinkReturn
+gst_avimux_audsinkconnect (GstPad *pad, GstCaps *vscaps)
+{
+  GstAviMux *avimux;
+  GstCaps *caps;
+
+  avimux = GST_AVIMUX (gst_pad_get_parent (pad));
+
+  /* we are not going to act on variable caps */
+  if (!GST_CAPS_IS_FIXED (vscaps))
+    return GST_PAD_LINK_DELAYED;
+
+  GST_DEBUG ("avimux: audio sinkconnect triggered on %s",
+	     gst_pad_get_name (pad));
 
   for (caps = vscaps; caps != NULL; caps = vscaps = vscaps->next)
   {
     const gchar* mimetype = gst_caps_get_mime(caps);
 
-    if (!strcmp (mimetype, "video/avi"))
-    {
-      const gchar* format;
-      
-      gst_caps_get_string (caps, "format", &format);
+    /* we want these for all */
+    gst_caps_get (caps, "channels", &avimux->auds.channels,
+			"rate",     &avimux->auds.rate,
+			NULL);
 
-      if (!strncmp (format, "strf_vids", 9)) {
-        avimux->vids.size        = sizeof(gst_riff_strf_vids);
-	gst_caps_get (caps,
-		      "width",       &avimux->vids.width,
-		      "height",      &avimux->vids.height,
-		      "planes",      &avimux->vids.planes,
-		      "bit_cnt",     &avimux->vids.bit_cnt,
-		      "compression", &avimux->vids.compression,
-		      "image_size",  &avimux->vids.image_size,
-		      "xpels_meter", &avimux->vids.xpels_meter,
-		      "ypels_meter", &avimux->vids.ypels_meter,
-		      "num_colors",  &avimux->vids.num_colors,
-		      "imp_colors",  &avimux->vids.imp_colors,
-		      NULL);
-        avimux->vids_hdr.fcc_handler = avimux->vids.compression;
-        avimux->avi_hdr.width = avimux->vids.width;
-        avimux->avi_hdr.height = avimux->vids.height;
-        goto done;
-      }
-      else if (!strncmp (format, "strf_auds", 9)) {
-	gst_caps_get (caps,
-		      "format",      &avimux->auds.format,
-		      "channels",    &avimux->auds.channels,
-		      "rate",        &avimux->auds.rate,
-		      "av_bps",      &avimux->auds.av_bps,
-		      "blockalign",  &avimux->auds.blockalign,
-		      "size",        &avimux->auds.size,
-		      NULL);
-        avimux->auds_hdr.samplesize = avimux->auds_hdr.scale = avimux->auds.blockalign;
-        avimux->auds_hdr.rate = avimux->auds.av_bps;
-        goto done;
-      }
-    }
-    else if (!strcmp (mimetype, "video/raw"))
+    if (!strcmp (mimetype, "audio/x-raw-int"))
     {
-      guint32 format;
-      gint temp;
-
-      gst_caps_get_fourcc_int (caps, "format", &format);
-      switch (format)
-      {
-        case GST_MAKE_FOURCC('Y','U','Y','2'):
-        case GST_MAKE_FOURCC('I','4','2','0'):
-        case GST_MAKE_FOURCC('Y','4','1','P'):
-        case GST_MAKE_FOURCC('R','G','B',' '):
-          avimux->vids.size        = sizeof(gst_riff_strf_vids);
-	  gst_caps_get (caps, "width", &avimux->vids.width,
-		              "height", &avimux->vids.height, NULL);
-          avimux->vids.planes      = 1;
-          switch (format)
-          {
-            case GST_MAKE_FOURCC('Y','U','Y','2'):
-              avimux->vids.bit_cnt     = 16; /* YUY2 */
-              break;
-            case GST_MAKE_FOURCC('R','G','B',' '):
-              gst_caps_get_int (caps, "bpp", &temp); /* RGB */
-	      avimux->vids.bit_cnt = temp;
-              break;
-            case GST_MAKE_FOURCC('Y','4','1','P'):
-            case GST_MAKE_FOURCC('I','4','2','0'):
-              avimux->vids.bit_cnt     = 12; /* Y41P or I420 */
-              break;
-          }
-          gst_caps_get_fourcc_int(caps, "format", &avimux->vids.compression);
-          avimux->vids_hdr.fcc_handler = avimux->vids.compression;
-          avimux->vids.image_size  = avimux->vids.height * avimux->vids.width;
-          avimux->avi_hdr.width = avimux->vids.width;
-          avimux->avi_hdr.height = avimux->vids.height;
-          goto done;
-        default:
-          break;
-      }
-    }
-    else if (!strcmp (mimetype, "video/jpeg"))
-    {
-      avimux->vids.size        = sizeof(gst_riff_strf_vids);
-      gst_caps_get (caps, "width", &avimux->vids.width,
-		          "height", &avimux->vids.height, NULL);
-      avimux->vids.planes      = 1;
-      avimux->vids.bit_cnt     = 24;
-      avimux->vids_hdr.fcc_handler = avimux->vids.compression = GST_MAKE_FOURCC('M','J','P','G');
-      avimux->avi_hdr.width = avimux->vids.width;
-      avimux->avi_hdr.height = avimux->vids.height;
-      avimux->vids.image_size  = avimux->vids.height * avimux->vids.width;
-      goto done;
-    }
-    else if (!strcmp (mimetype, "audio/raw"))
-    {
-      gint width;
-
       avimux->auds.format      = GST_RIFF_WAVE_FORMAT_PCM;
-      gst_caps_get (caps, "channels", 	&avimux->auds.channels,
-		          "rate", 	&avimux->auds.rate,
-		          "width",	&width,
+
+      gst_caps_get (caps, "width",	&avimux->auds.blockalign,
 		          "depth",	&avimux->auds.size,
 		          NULL);
-      avimux->auds_hdr.rate = avimux->auds.av_bps = width * avimux->auds.rate * avimux->auds.channels / 8;
-      avimux->auds_hdr.samplesize = avimux->auds_hdr.scale = avimux->auds.blockalign = width * avimux->auds.channels/8;
+
+      /* set some more info straight */
+      avimux->auds.blockalign /= 8;
+      avimux->auds.blockalign *= avimux->auds.channels;
+      avimux->auds.av_bps = avimux->auds.blockalign * avimux->auds.rate;
       goto done;
     }
-    else if (!strcmp (mimetype, "audio/x-mp3"))
+    else if (!strcmp (mimetype, "audio/mpeg") ||
+	     !strcmp (mimetype, "audio/x-vorbis") ||
+	     !strcmp (mimetype, "audio/x-ac3"))
     {
-      gint layer;
+      avimux->auds.format = 0;
 
-      gst_caps_get_int(caps, "layer", &layer);
+      if (!strcmp (mimetype, "audio/mpeg")) {
+        gint layer = 3;
+        gst_caps_get_int(caps, "layer", &layer);
+        switch (layer) {
+          case 3:
+            avimux->auds.format = GST_RIFF_WAVE_FORMAT_MPEGL3;
+            break;
+          case 1: case 2:
+            avimux->auds.format = GST_RIFF_WAVE_FORMAT_MPEGL12;
+            break;
+        }
+      } else if (!strcmp (mimetype, "audio/x-vorbis")) {
+        avimux->auds.format = GST_RIFF_WAVE_FORMAT_VORBIS3;
+      } else if (!strcmp (mimetype, "audio/x-ac3")) {
+        avimux->auds.format = GST_RIFF_WAVE_FORMAT_A52;
+      }
+
+      avimux->auds.blockalign = 1;
+      avimux->auds.av_bps = 0;
+      avimux->auds.size = 16;
+
+      if (!avimux->auds.format) {
+        continue;
+      }
 
-      /* we don't need to do anything here, compressed mp3 contains it all */
-      avimux->auds.format      = (layer == 3?
-                                   GST_RIFF_WAVE_FORMAT_MPEGL3 : 
-				   GST_RIFF_WAVE_FORMAT_MPEGL12);
-      goto done;
-    }
-    else if (!strcmp (mimetype, "application/x-ogg"))
-    {
-      avimux->auds.format = GST_RIFF_WAVE_FORMAT_VORBIS1;
       goto done;
     }
   }
-  return GST_PAD_CONNECT_REFUSED;
+  return GST_PAD_LINK_REFUSED;
 
 done:
-  return GST_PAD_CONNECT_OK;
+  avimux->auds_hdr.rate = avimux->auds.blockalign * avimux->auds.rate; 
+  avimux->auds_hdr.samplesize = avimux->auds.blockalign;
+  avimux->auds_hdr.scale = avimux->auds.blockalign;
+  return GST_PAD_LINK_OK;
 }
 
 static void
-gst_avimux_pad_connect (GstPad   *pad,
-                        GstPad   *peer,
-                        gpointer  data)
+gst_avimux_pad_link (GstPad   *pad,
+                     GstPad   *peer,
+                     gpointer  data)
 {
   GstAviMux *avimux = GST_AVIMUX(data);
   const gchar *padname = gst_pad_get_name (pad);
@@ -463,13 +541,13 @@ gst_avimux_pad_connect (GstPad   *pad,
     return;
   }
 
-  GST_DEBUG(GST_CAT_PLUGIN_INFO, "pad '%s' connected", padname);
+  GST_DEBUG ("pad '%s' connected", padname);
 }
 
 static void
-gst_avimux_pad_disconnect (GstPad   *pad,
-                           GstPad   *peer,
-                           gpointer  data)
+gst_avimux_pad_unlink (GstPad   *pad,
+                       GstPad   *peer,
+                       gpointer  data)
 {
   GstAviMux *avimux = GST_AVIMUX(data);
   const gchar *padname = gst_pad_get_name (pad);
@@ -490,10 +568,7 @@ gst_avimux_pad_disconnect (GstPad   *pad,
     return;
   }
 
-  GST_DEBUG(GST_CAT_PLUGIN_INFO, "pad '%s' disconnected", padname);
-
-  /* is this allowed? */
-  gst_pad_destroy(pad);
+  GST_DEBUG ("pad '%s' unlinked", padname);
 }
 
 static GstPad*
@@ -518,11 +593,13 @@ gst_avimux_request_new_pad (GstElement     *element,
   if (templ == GST_PAD_TEMPLATE_GET (audio_sink_factory)) {
     g_return_val_if_fail(avimux->audiosinkpad == NULL, NULL);
     newpad = gst_pad_new_from_template (templ, "audio_00");
+    gst_pad_set_link_function (newpad, gst_avimux_audsinkconnect);
     avimux->audiosinkpad = newpad;
   }
   else if (templ == GST_PAD_TEMPLATE_GET (video_sink_factory)) {
     g_return_val_if_fail(avimux->videosinkpad == NULL, NULL);
     newpad = gst_pad_new_from_template (templ, "video_00");
+    gst_pad_set_link_function (newpad, gst_avimux_vidsinkconnect);
     avimux->videosinkpad = newpad;
   }
   else {
@@ -530,11 +607,10 @@ gst_avimux_request_new_pad (GstElement     *element,
     return NULL;
   }
 
-  g_signal_connect(newpad, "connected",
-    G_CALLBACK(gst_avimux_pad_connect), (gpointer)avimux);
-  g_signal_connect(newpad, "disconnected",
-    G_CALLBACK(gst_avimux_pad_disconnect), (gpointer)avimux);
-  gst_pad_set_connect_function (newpad, gst_avimux_sinkconnect);
+  g_signal_connect(newpad, "linked",
+    G_CALLBACK(gst_avimux_pad_link), (gpointer)avimux);
+  g_signal_connect(newpad, "unlinked",
+    G_CALLBACK(gst_avimux_pad_unlink), (gpointer)avimux);
   gst_element_add_pad (element, newpad);
   gst_pad_set_event_function(newpad, gst_avimux_handle_event);
   gst_pad_set_event_mask_function(newpad, gst_avimux_get_event_masks);
@@ -830,7 +906,6 @@ gst_avimux_add_index (GstAviMux *avimux, guchar *code, guint32 flags, guint32 si
   avimux->idx[avimux->idx_index].offset = LE_FROM_GUINT32(avimux->idx_offset);
   avimux->idx[avimux->idx_index].size = LE_FROM_GUINT32(size);
   avimux->idx_index++;
-  avimux->idx_offset += size;
 }
 
 static void
@@ -845,14 +920,14 @@ gst_avimux_write_index (GstAviMux *avimux)
   memcpy(GST_BUFFER_DATA(buffer), "idx1", 4);
   temp32 = LE_FROM_GUINT32(avimux->idx_index * sizeof(gst_riff_index_entry)); 
   memcpy(GST_BUFFER_DATA(buffer)+4, &temp32, 4);
-  gst_pad_push(avimux->srcpad, buffer);
+  gst_pad_push(avimux->srcpad, GST_DATA (buffer));
 
   buffer = gst_buffer_new();
   GST_BUFFER_SIZE(buffer) = avimux->idx_index * sizeof(gst_riff_index_entry);
   GST_BUFFER_DATA(buffer) = (unsigned char*) avimux->idx;
   avimux->idx = NULL; /* will be free()'ed by gst_buffer_unref() */
   avimux->total_data += GST_BUFFER_SIZE(buffer);
-  gst_pad_push(avimux->srcpad, buffer);
+  gst_pad_push(avimux->srcpad, GST_DATA (buffer));
 
   avimux->idx_size += avimux->idx_index * sizeof(gst_riff_index_entry) + 8;
 
@@ -874,19 +949,18 @@ gst_avimux_bigfile(GstAviMux *avimux, gboolean last)
 				GST_SEEK_FLAG_FLUSH, 
 				avimux->avix_start);
     /* if the event succeeds */
-    if (gst_pad_send_event(GST_PAD_PEER(avimux->srcpad), event)) {
+    gst_pad_push(avimux->srcpad, GST_DATA(event));
 
-      /* rewrite AVIX header */
-      header = gst_avimux_riff_get_avix_header(avimux->datax_size);
-      gst_pad_push(avimux->srcpad, header);
+    /* rewrite AVIX header */
+    header = gst_avimux_riff_get_avix_header(avimux->datax_size);
+    gst_pad_push(avimux->srcpad, GST_DATA (header));
 
-      /* go back to current location */
-      event = gst_event_new_seek (GST_FORMAT_BYTES | 
-		    	          GST_SEEK_METHOD_SET | 
-				  GST_SEEK_FLAG_FLUSH, 
-				  avimux->total_data);
-      gst_pad_send_event(GST_PAD_PEER(avimux->srcpad), event);
-    }
+    /* go back to current location */
+    event = gst_event_new_seek (GST_FORMAT_BYTES | 
+				GST_SEEK_METHOD_SET |
+				GST_SEEK_FLAG_FLUSH, 
+				avimux->total_data);
+    gst_pad_push(avimux->srcpad, GST_DATA(event));
   }
   avimux->avix_start = avimux->total_data;
 
@@ -899,7 +973,7 @@ gst_avimux_bigfile(GstAviMux *avimux, gboolean last)
 
   header = gst_avimux_riff_get_avix_header(0);
   avimux->total_data += GST_BUFFER_SIZE(header);
-  gst_pad_push(avimux->srcpad, header);
+  gst_pad_push(avimux->srcpad, GST_DATA (header));
 }
 
 /* enough header blabla now, let's go on to actually writing the headers */
@@ -916,6 +990,7 @@ gst_avimux_start_file (GstAviMux *avimux)
   avimux->num_frames = 0;
   avimux->numx_frames = 0;
   avimux->audio_size = 0;
+  avimux->audio_time = 0;
   avimux->avix_start = 0;
 
   avimux->idx_index = 0;
@@ -925,13 +1000,13 @@ gst_avimux_start_file (GstAviMux *avimux)
   avimux->idx = NULL;
 
   /* header */
-  avimux->avi_hdr.streams = avimux->video_pad_connected?1:0 + avimux->audio_pad_connected?1:0;
+  avimux->avi_hdr.streams = (avimux->video_pad_connected?1:0) + (avimux->audio_pad_connected?1:0);
   avimux->is_bigfile = FALSE;
 
   header = gst_avimux_riff_get_avi_header(avimux);
-  avimux->idx_offset = avimux->header_size + 12;
   avimux->total_data += GST_BUFFER_SIZE(header);
-  gst_pad_push(avimux->srcpad, header);
+  avimux->idx_offset = avimux->total_data;
+  gst_pad_push(avimux->srcpad, GST_DATA (header));
 
   avimux->write_header = FALSE;
   avimux->restart = FALSE;
@@ -959,33 +1034,43 @@ gst_avimux_stop_file (GstAviMux *avimux)
 
   /* statistics/total_frames/... */
   avimux->avi_hdr.tot_frames = avimux->num_frames;
-  if (avimux->video_pad_connected)
+  if (avimux->video_pad_connected) {
     avimux->vids_hdr.length = avimux->num_frames;
-  if (avimux->audio_pad_connected)
-  {
-    if (avimux->auds_hdr.scale)
-      avimux->auds_hdr.length = avimux->audio_size/avimux->auds_hdr.scale;
-    else
-      avimux->auds_hdr.length = 0; /* urm...? FIXME! ;-) */
+  }
+  if (avimux->audio_pad_connected) {
+    avimux->auds_hdr.length = (avimux->audio_time * avimux->auds.rate)/GST_SECOND;
   }
 
   /* set rate and everything having to do with that */
-  avimux->avi_hdr.us_frame = avimux->vids_hdr.scale = 1000000/avimux->framerate;
   avimux->avi_hdr.max_bps = 0;
-  if (avimux->audio_pad_connected)
+  if (avimux->audio_pad_connected) {
+    /* calculate bps if needed */
+    if (!avimux->auds.av_bps) {
+      if (avimux->audio_time) {
+        avimux->auds_hdr.rate = (GST_SECOND * avimux->audio_size) / avimux->audio_time;
+      } else {
+        gst_element_error (GST_ELEMENT (avimux), "Audio stream available, but no audio data transferred (or data with invalid timestamps). Resulting AVI will be corrupt");
+        avimux->auds_hdr.rate = 0;
+      }
+      avimux->auds.av_bps = avimux->auds_hdr.rate * avimux->auds_hdr.scale;
+    }
     avimux->avi_hdr.max_bps += avimux->auds.av_bps;
-  if (avimux->video_pad_connected)
+  }
+  if (avimux->video_pad_connected) {
     avimux->avi_hdr.max_bps += ((avimux->vids.bit_cnt+7)/8) *
-				avimux->framerate *
+				(1000000. / avimux->avi_hdr.us_frame) *
 				avimux->vids.image_size;
+  }
 
   /* seek and rewrite the header */
   header = gst_avimux_riff_get_avi_header(avimux);
   event = gst_event_new_seek (GST_FORMAT_BYTES | 
-		  	      GST_SEEK_METHOD_SET |
-			      GST_SEEK_FLAG_FLUSH, 0);
-  gst_pad_send_event(GST_PAD_PEER(avimux->srcpad), event);
-  gst_pad_push(avimux->srcpad, header);
+			      GST_SEEK_METHOD_SET, 0);
+  gst_pad_push(avimux->srcpad, GST_DATA(event));
+  gst_pad_push(avimux->srcpad, GST_DATA (header));
+  event = gst_event_new_seek (GST_FORMAT_BYTES |
+			      GST_SEEK_METHOD_SET, avimux->total_data);
+  gst_pad_push(avimux->srcpad, GST_DATA(event));
 
   avimux->write_header = TRUE;
 }
@@ -997,8 +1082,8 @@ gst_avimux_restart_file (GstAviMux *avimux)
 
   gst_avimux_stop_file(avimux);
 
-  event = gst_event_new(GST_EVENT_NEW_MEDIA);
-  gst_pad_send_event(avimux->srcpad, event);
+  event = gst_event_new(GST_EVENT_EOS);
+  gst_pad_push(avimux->srcpad, GST_DATA(event));
 
   gst_avimux_start_file(avimux);
 }
@@ -1015,17 +1100,15 @@ gst_avimux_handle_event (GstPad *pad, GstEvent *event)
   type = event ? GST_EVENT_TYPE (event) : GST_EVENT_UNKNOWN;
 
   switch (type) {
-    case GST_EVENT_NEW_MEDIA:
-      avimux->restart = TRUE;
-      break;
     case GST_EVENT_EOS:
       /* is this allright? */
-      if (pad == avimux->videosinkpad)
+      if (pad == avimux->videosinkpad) {
         avimux->video_pad_eos = TRUE;
-      else if (pad == avimux->audiosinkpad)
+      } else if (pad == avimux->audiosinkpad) {
         avimux->audio_pad_eos = TRUE;
-      else
+      } else {
         g_warning("Unknown pad for EOS!");
+      }
       break;
     default:
       break;
@@ -1041,43 +1124,33 @@ gst_avimux_fill_queue (GstAviMux *avimux)
 {
   GstBuffer *buffer;
 
-  if (!avimux->audio_buffer_queue &&
-       GST_PAD_IS_USABLE(avimux->audiosinkpad) &&
-      !avimux->audio_pad_eos)
+  while (!avimux->audio_buffer_queue &&
+          avimux->audiosinkpad &&
+          avimux->audio_pad_connected &&
+          GST_PAD_IS_USABLE(avimux->audiosinkpad) &&
+         !avimux->audio_pad_eos)
   {
-    while (1)
-    {
-      buffer = gst_pad_pull(avimux->audiosinkpad);
-      if (GST_IS_EVENT(buffer))
-      {
-        gst_avimux_handle_event(avimux->audiosinkpad, GST_EVENT(buffer));
-      }
-      else
-      {
-        avimux->audio_buffer_queue = buffer;
-        break;
-      }
+    buffer = GST_BUFFER (gst_pad_pull(avimux->audiosinkpad));
+    if (GST_IS_EVENT(buffer)) {
+      gst_avimux_handle_event(avimux->audiosinkpad, GST_EVENT(buffer));
+    } else {
+      avimux->audio_buffer_queue = buffer;
+      break;
     }
   }
 
-  if (!avimux->video_buffer_queue &&
-       GST_PAD_IS_USABLE(avimux->videosinkpad) &&
-      !avimux->video_pad_eos)
+  while (!avimux->video_buffer_queue &&
+          avimux->videosinkpad &&
+          avimux->video_pad_connected &&
+          GST_PAD_IS_USABLE(avimux->videosinkpad) &&
+         !avimux->video_pad_eos)
   {
-    while (1)
-    {
-      buffer = gst_pad_pull(avimux->videosinkpad);
-      if (GST_IS_EVENT(buffer))
-      {
-        gst_avimux_handle_event(avimux->videosinkpad, GST_EVENT(buffer));
-      }
-      else
-      {
-        avimux->video_buffer_queue = buffer;
-	if (avimux->framerate < 0)
-          avimux->framerate = gst_video_frame_rate(GST_PAD_PEER(avimux->videosinkpad));
-        break;
-      }
+    buffer = GST_BUFFER (gst_pad_pull(avimux->videosinkpad));
+    if (GST_IS_EVENT(buffer)) {
+      gst_avimux_handle_event(avimux->videosinkpad, GST_EVENT(buffer));
+    } else {
+      avimux->video_buffer_queue = buffer;
+      break;
     }
   }
 }
@@ -1095,7 +1168,7 @@ gst_avimux_send_pad_data (GstAviMux *avimux,
   GST_BUFFER_DATA(buffer) = g_malloc(num_bytes);
   memset(GST_BUFFER_DATA(buffer), 0, num_bytes);
 
-  gst_pad_push(avimux->srcpad, buffer);
+  gst_pad_push(avimux->srcpad, GST_DATA (buffer));
 }
 
 /* do audio buffer */
@@ -1103,12 +1176,14 @@ static void
 gst_avimux_do_audio_buffer (GstAviMux *avimux)
 {
   GstBuffer *data = avimux->audio_buffer_queue, *header;
-  gulong total_size, pad_bytes;
+  gulong total_size, pad_bytes = 0;
 
   /* write a audio header + index entry */
-  header = gst_avimux_riff_get_audio_header((GST_BUFFER_SIZE(data)+3)&~3);
-  total_size = GST_BUFFER_SIZE(header) + GST_BUFFER_SIZE(data);
-  avimux->total_data += total_size;
+  if (GST_BUFFER_SIZE(data) & 1) {
+    pad_bytes = 2 - (GST_BUFFER_SIZE(data) & 1);
+  }
+  header = gst_avimux_riff_get_audio_header(GST_BUFFER_SIZE(data));
+  total_size = GST_BUFFER_SIZE(header) + GST_BUFFER_SIZE(data) + pad_bytes;
 
   if (avimux->is_bigfile)
   {
@@ -1118,20 +1193,18 @@ gst_avimux_do_audio_buffer (GstAviMux *avimux)
   {
     avimux->data_size += total_size;
     avimux->audio_size += GST_BUFFER_SIZE(data);
-    gst_avimux_add_index(avimux, "01wb", 0x0, total_size);
+    avimux->audio_time += GST_BUFFER_DURATION(data);
+    gst_avimux_add_index(avimux, "01wb", 0x0, GST_BUFFER_SIZE(data));
   }
 
-  gst_pad_push(avimux->srcpad, header);
-  pad_bytes = ((GST_BUFFER_SIZE(data)+3)&~3) - GST_BUFFER_SIZE(data);
-  if (pad_bytes)
-    if (GST_BUFFER_MAXSIZE(data) >= GST_BUFFER_SIZE(data) + pad_bytes)
-    {
-      GST_BUFFER_SIZE(data) += pad_bytes;
-      pad_bytes = 0;
-    }
-  gst_pad_push(avimux->srcpad, data);
-  if (pad_bytes)
+  gst_pad_push(avimux->srcpad, GST_DATA (header));
+  gst_pad_push(avimux->srcpad, GST_DATA (data));
+  if (pad_bytes) {
     gst_avimux_send_pad_data(avimux, pad_bytes);
+  }
+  avimux->total_data += total_size;
+  avimux->idx_offset += total_size;
+
   avimux->audio_buffer_queue = NULL;
 }
 
@@ -1141,7 +1214,7 @@ static void
 gst_avimux_do_video_buffer (GstAviMux *avimux)
 {
   GstBuffer *data = avimux->video_buffer_queue, *header;
-  gulong total_size, pad_bytes;
+  gulong total_size, pad_bytes = 0;
 
   if (avimux->restart)
     gst_avimux_restart_file(avimux);
@@ -1155,9 +1228,11 @@ gst_avimux_do_video_buffer (GstAviMux *avimux)
       gst_avimux_restart_file(avimux);
   }
 
-  header = gst_avimux_riff_get_video_header((GST_BUFFER_SIZE(data)+3)&~3);
-  total_size = GST_BUFFER_SIZE(header) + GST_BUFFER_SIZE(data);
-  avimux->total_data += total_size;
+  if (GST_BUFFER_SIZE(data) & 1) {
+    pad_bytes = 2 - (GST_BUFFER_SIZE(data) & 1);
+  }
+  header = gst_avimux_riff_get_video_header(GST_BUFFER_SIZE(data));
+  total_size = GST_BUFFER_SIZE(header) + GST_BUFFER_SIZE(data) + pad_bytes;
   avimux->total_frames++;
 
   if (avimux->is_bigfile)
@@ -1167,22 +1242,22 @@ gst_avimux_do_video_buffer (GstAviMux *avimux)
   }
   else
   {
+    guint flags = 0x2;
+    if (GST_BUFFER_FLAG_IS_SET (data, GST_BUFFER_KEY_UNIT))
+      flags |= 0x10;
     avimux->data_size += total_size;
     avimux->num_frames++;
-    gst_avimux_add_index(avimux, "00db", 0x12, total_size);
+    gst_avimux_add_index(avimux, "00db", flags, GST_BUFFER_SIZE(data));
   }
 
-  gst_pad_push(avimux->srcpad, header);
-  pad_bytes = ((GST_BUFFER_SIZE(data)+3)&~3) - GST_BUFFER_SIZE(data);
-  if (pad_bytes)
-    if (GST_BUFFER_MAXSIZE(data) >= GST_BUFFER_SIZE(data) + pad_bytes)
-    {
-      GST_BUFFER_SIZE(data) += pad_bytes;
-      pad_bytes = 0;
-    }
-  gst_pad_push(avimux->srcpad, data);
-  if (pad_bytes)
+  gst_pad_push(avimux->srcpad, GST_DATA (header));
+  gst_pad_push(avimux->srcpad, GST_DATA (data));
+  if (pad_bytes) {
     gst_avimux_send_pad_data(avimux, pad_bytes);
+  }
+  avimux->total_data += total_size;
+  avimux->idx_offset += total_size;
+
   avimux->video_buffer_queue = NULL;
 }
 
@@ -1208,8 +1283,14 @@ gst_avimux_do_one_buffer (GstAviMux *avimux)
     else
       gst_avimux_do_audio_buffer(avimux);
   }
-  else
+  else {
+    /* simply finish off the file and send EOS */
+    gst_avimux_stop_file(avimux);
+    gst_pad_push(avimux->srcpad,
+                 GST_DATA(gst_event_new(GST_EVENT_EOS)));
+    gst_element_set_eos(GST_ELEMENT(avimux));
     return FALSE;
+  }
 
   return TRUE;
 }
@@ -1289,19 +1370,9 @@ gst_avimux_change_state (GstElement *element)
   avimux = GST_AVIMUX(element);
 
   switch (transition) {
-    case GST_STATE_READY_TO_PAUSED:
-      break;
     case GST_STATE_PAUSED_TO_PLAYING:
-      avimux->framerate = -1; /* means that we fill it in later */
       avimux->video_pad_eos = avimux->audio_pad_eos = FALSE;
       break;
-    case GST_STATE_PLAYING_TO_PAUSED:
-      /* this function returns TRUE while it handles buffers */
-      while (gst_avimux_do_one_buffer(avimux));
-      gst_avimux_stop_file(avimux);
-      break;
-    case GST_STATE_PAUSED_TO_READY:
-      break;
   }
 
   if (GST_ELEMENT_CLASS (parent_class)->change_state)
