@@ -17,8 +17,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 #include <gstsmooth.h>
+#include <gst/video/video.h>
 
 /* elementfactory information */
 static GstElementDetails smooth_details = {
@@ -50,10 +54,11 @@ GST_PAD_TEMPLATE_FACTORY (smooth_src_factory,
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
+  gst_caps_new (
    "smooth_src",
-   "video/raw",
-     "format",   GST_PROPS_FOURCC (GST_STR_FOURCC ("I420"))
+   "video/x-raw-yuv",
+     GST_VIDEO_YUV_PAD_TEMPLATE_PROPS(
+	     GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")))
   )
 )
 
@@ -61,17 +66,18 @@ GST_PAD_TEMPLATE_FACTORY (smooth_sink_factory,
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
+  gst_caps_new (
    "smooth_src",
-   "video/raw",
-     "format",   GST_PROPS_FOURCC (GST_STR_FOURCC ("I420"))
+   "video/x-raw-yuv",
+     GST_VIDEO_YUV_PAD_TEMPLATE_PROPS(
+	     GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")))
   )
 )
 
 static void	gst_smooth_class_init	(GstSmoothClass *klass);
 static void	gst_smooth_init		(GstSmooth *smooth);
 
-static void	gst_smooth_chain	(GstPad *pad, GstBuffer *buf);
+static void	gst_smooth_chain	(GstPad *pad, GstData *_data);
 static void	smooth_filter		(unsigned char* dest, unsigned char* src,
 					 int width, int height, int tolerance, int filtersize);
 
@@ -141,7 +147,7 @@ gst_smooth_sinkconnect (GstPad *pad, GstCaps *caps)
   gst_caps_get_int (caps, "width", &filter->width);
   gst_caps_get_int (caps, "height", &filter->height);
 
-  return GST_PAD_LINK_OK;
+  return gst_pad_try_set_caps (filter->srcpad, caps);
 }
 
 static void
@@ -212,8 +218,9 @@ smooth_filter (unsigned char* dest, unsigned char* src, int width, int height, i
 }
 
 static void
-gst_smooth_chain (GstPad *pad, GstBuffer *buf)
+gst_smooth_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstSmooth *smooth;
   guchar *data;
   gulong size;
@@ -227,14 +234,14 @@ gst_smooth_chain (GstPad *pad, GstBuffer *buf)
   smooth = GST_SMOOTH (GST_OBJECT_PARENT (pad));
 
   if (!smooth->active) {
-    gst_pad_push(smooth->srcpad,buf);
+    gst_pad_push(smooth->srcpad,GST_DATA (buf));
     return;
   }
 
   data = GST_BUFFER_DATA (buf);
   size = GST_BUFFER_SIZE (buf);
 
-  GST_DEBUG (0,"smooth: have buffer of %d", GST_BUFFER_SIZE (buf));
+  GST_DEBUG ("smooth: have buffer of %d", GST_BUFFER_SIZE (buf));
 
   outbuf = gst_buffer_new();
   GST_BUFFER_DATA (outbuf) = g_malloc (GST_BUFFER_SIZE (buf));
@@ -259,7 +266,7 @@ gst_smooth_chain (GstPad *pad, GstBuffer *buf)
 
   gst_buffer_unref (buf);
 
-  gst_pad_push (smooth->srcpad, outbuf);
+  gst_pad_push (smooth->srcpad, GST_DATA (outbuf));
 }
 
 static void
