@@ -41,6 +41,7 @@ extern "C" {
 #include <sys/socket.h>
 #include <sys/wait.h>
 #include <fcntl.h>
+#include <arpa/inet.h>
 #include "gstudp.h"
 
 #define GST_TYPE_UDPSINK \
@@ -71,11 +72,14 @@ struct _GstUDPSink {
 
   int sock;
   struct sockaddr_in theiraddr;
+  struct ip_mreq multi_addr;
 
   gint port;
   Gst_UDP_Control control;
   gchar *host;
     
+  guint mtu;
+    
   GstClock *clock;
 };
 
