@@ -45,28 +45,59 @@ struct _GstV4l2Src {
 	GstPad *srcpad;
 
 	/* internal lists */
-	GList /*v4l2_fmtdesc*/ *formats; /* list of available capture formats */
+	GList /*v4l2_fmtdesc*/ *formats, *format_list; /* list of available capture formats */
 
 	/* buffer properties */
 	struct v4l2_buffer bufsettings;
 	struct v4l2_requestbuffers breq;
 	struct v4l2_format format;
-	guint64 first_timestamp;
+
+	/* num of queued frames and some GThread stuff
+	 * to wait if there's not enough */
+	gint8 *frame_queue_state;
+	GMutex *mutex_queue_state;
+	GCond *cond_queue_state;
+	gint num_queued;
+	gint queue_frame;
+
+	/* True if we want to stop */
+	gboolean quit;
+
+	/* A/V sync... frame counter and internal cache */
+	gulong handled;
+	gint last_frame;
+	gint need_writes;
+	gulong last_seq;
+
+	/* clock */
+	GstClock *clock;
+	
+	/* time to substract from clock time to get back to timestamp */
+	GstClockTime substract_time;
+
+	/* how often are we going to use each frame? */
+	gint *use_num_times;
+
+	/* how are we going to push buffers? */
+	gboolean use_fixed_fps;
 
 	/* bufferpool for the buffers we're gonna use */
 	GstBufferPool *bufferpool;
-
-	/* caching values */
-	gint width;
-	gint height;
-	gint palette;
 };
 
 struct _GstV4l2SrcClass {
 	GstV4l2ElementClass parent_class;
+
+	void (*frame_capture) (GObject *object);
+	void (*frame_drop)    (GObject *object);
+	void (*frame_insert)  (GObject *object);
+	void (*frame_lost)    (GObject *object,
+                               gint     num_lost);
 };
 
 
 GType gst_v4l2src_get_type(void);
 
+gboolean gst_v4l2src_factory_init (GstPlugin *plugin);
+
 #endif /* __GST_V4L2SRC_H__ */
