@@ -1,6 +1,34 @@
-/* Based on xqcam.c by Paul Chinn <loomer@svpal.org> */
+/* GStreamer
+ * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
 
+/* Based on xqcam.c by Paul Chinn <loomer@svpal.org>:
+ *
+ *  XWindow Support for QuickCam
+ *  by Paul Chinn <loomer@svpal.org>
+ *  Modified by Scott Laird <scott@laird.com>
+ *
+ * xqcam.c is under the X license.
+ */
+
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 /* gcc -ansi -pedantic on GNU/Linux causes warnings and errors
  * unless this is defined:
@@ -24,12 +52,17 @@
 #include <X11/Shell.h>
 
 #include "gstxwindow.h"
+#include "gstximage.h"
+#ifdef HAVE_XVIDEO
+#include "gstxvimage.h"
+#endif
 
 GstXWindow *
 _gst_xwindow_new (gint width, gint height, gboolean toplevel)
 {
   XGCValues values;
   GstXWindow *new;
+  int i;
 
   new = g_new0 (GstXWindow, 1);
 
@@ -71,8 +104,25 @@ _gst_xwindow_new (gint width, gint height, gboolean toplevel)
     XMapRaised (new->disp, new->win);
   }
 
+  new->image_formats = g_ptr_array_new();
+
+#ifdef HAVE_XVIDEO
+  _gst_xvimage_add_formats(new);
+#endif
+  _gst_ximage_add_formats(new);
+
+  for(i=0;i<new->image_formats->len;i++){
+    GstImageFormat *image_format;
+
+    image_format = g_ptr_array_index(new->image_formats,i);
+
+    GST_DEBUG ("%d: %d %s",i,image_format->type,
+	gst_caps_to_string(image_format->caps));
+  }
+
   return new;
 }
+
 static inline void
 gst_xwindow_real_destroy (GstXWindow * window)
 {
@@ -82,6 +132,7 @@ gst_xwindow_real_destroy (GstXWindow * window)
     g_free (window);
   }
 }
+
 void
 _gst_xwindow_destroy (GstXWindow * window)
 {
@@ -89,11 +140,13 @@ _gst_xwindow_destroy (GstXWindow * window)
   XDestroyWindow (window->disp, window->win);
   gst_xwindow_real_destroy (window);
 }
+
 void
 _gst_xwindow_ref (GstXWindow * window)
 {
   window->imagecount++;
 }
+
 void
 _gst_xwindow_unref (GstXWindow * window)
 {
